@@ -23,13 +23,16 @@
 #include "config.h"
 
 #include <glib/gi18n.h>
+
 #include <libpeas/peas.h>
 
 #include "ide-context.h"
 #include "ide-context-private.h"
-#include "ide-context-addin.h"
+#include "ide-debug.h"
+#include "ide-gsettings-action-group.h"
 #include "ide-macros.h"
 #include "ide-notifications.h"
+#include "ide-settings.h"
 
 /**
  * SECTION:ide-context
@@ -38,18 +41,16 @@
  *
  * The #IdeContext object is the root object for a project. Everything
  * in a project is contained by this object.
- *
- * Since: 3.32
  */
 
 struct _IdeContext
 {
-  IdeObject         parent_instance;
-  PeasExtensionSet *addins;
-  gchar            *project_id;
-  gchar            *title;
-  GFile            *workdir;
-  guint             project_loaded : 1;
+  IdeObject       parent_instance;
+  char           *project_id;
+  char           *title;
+  GFile          *workdir;
+  IdeActionMuxer *action_muxer;
+  guint           project_loaded : 1;
 };
 
 enum {
@@ -69,71 +70,19 @@ G_DEFINE_FINAL_TYPE (IdeContext, ide_context, IDE_TYPE_OBJECT)
 
 static GParamSpec *properties [N_PROPS];
 static guint signals [N_SIGNALS];
-
-static void
-ide_context_addin_load_project_cb (GObject      *object,
-                                   GAsyncResult *result,
-                                   gpointer      user_data)
-{
-  IdeContextAddin *addin = (IdeContextAddin *)object;
-  g_autoptr(IdeContext) self = user_data;
-  g_autoptr(GError) error = NULL;
-
-  g_assert (IDE_IS_CONTEXT_ADDIN (addin));
-  g_assert (G_IS_ASYNC_RESULT (result));
-  g_assert (IDE_IS_CONTEXT (self));
-
-  if (ide_context_addin_load_project_finish (addin, result, &error))
-    ide_context_addin_project_loaded (addin, self);
-  else
-    g_warning ("%s context addin failed to load project: %s",
-               G_OBJECT_TYPE_NAME (addin), error->message);
-}
-
-static void
-ide_context_addin_added_cb (PeasExtensionSet *set,
-                            PeasPluginInfo   *plugin_info,
-                            PeasExtension    *exten,
-                            gpointer          user_data)
-{
-  IdeContextAddin *addin = (IdeContextAddin *)exten;
-  IdeContext *self = user_data;
-  g_autoptr(GCancellable) cancellable = NULL;
-
-  g_assert (PEAS_IS_EXTENSION_SET (set));
-  g_assert (plugin_info != NULL);
-  g_assert (IDE_IS_CONTEXT_ADDIN (addin));
-
-  /* Ignore any request during shutdown */
-  cancellable = ide_object_ref_cancellable (IDE_OBJECT (self));
-  if (g_cancellable_is_cancelled (cancellable))
-    return;
-
-  ide_context_addin_load (addin, self);
-
-  if (self->project_loaded)
-    ide_context_addin_load_project_async (addin,
-                                          self,
-                                          cancellable,
-                                          ide_context_addin_load_project_cb,
-                                          g_object_ref (self));
-}
-
-static void
-ide_context_addin_removed_cb (PeasExtensionSet *set,
-                              PeasPluginInfo   *plugin_info,
-                              PeasExtension    *exten,
-                              gpointer          user_data)
-{
-  IdeContextAddin *addin = (IdeContextAddin *)exten;
-  IdeContext *self = user_data;
-
-  g_assert (PEAS_IS_EXTENSION_SET (set));
-  g_assert (plugin_info != NULL);
-  g_assert (IDE_IS_CONTEXT_ADDIN (addin));
-
-  ide_context_addin_unload (addin, self);
-}
+static const char *app_schema_ids[] = {
+  "org.gnome.builder",
+  "org.gnome.builder.code-insight",
+  "org.gnome.builder.editor",
+  "org.gnome.builder.project-tree",
+  "org.gnome.builder.spelling",
+  "org.gnome.builder.terminal",
+};
+static const char *project_schema_ids[] = {
+  "org.gnome.builder.build",
+  "org.gnome.builder.debug",
+  "org.gnome.builder.project",
+};
 
 static void
 ide_context_real_log (IdeContext     *self,
@@ -155,32 +104,98 @@ ide_context_repr (IdeObject *object)
                           self->project_loaded);
 }
 
+static void
+ide_context_engine_load_plugin_cb (IdeContext           *self,
+                                   const PeasPluginInfo *plugin_info,
+                                   PeasEngine           *engine)
+{
+  g_autofree char *schemas = NULL;
+
+  g_assert (IDE_IS_CONTEXT (self));
+  g_assert (plugin_info != NULL);
+  g_assert (PEAS_IS_ENGINE (engine));
+
+  if ((schemas = g_strdup (peas_plugin_info_get_external_data (plugin_info, "Settings-Schemas"))))
+    {
+      g_auto(GStrv) split = g_strsplit (g_strdelimit (schemas, " ,\t:", ';'), ";", 0);
+
+      for (guint i = 0; split[i]; i++)
+        {
+          g_strstrip (split[i]);
+
+          if (!ide_str_empty0 (split[i]))
+            ide_context_register_settings (self, split[i]);
+        }
+    }
+}
+
+static void
+ide_context_engine_unload_plugin_cb (IdeContext           *self,
+                                     const PeasPluginInfo *plugin_info,
+                                     PeasEngine           *engine)
+{
+  g_autofree char *schemas = NULL;
+
+  g_assert (IDE_IS_CONTEXT (self));
+  g_assert (plugin_info != NULL);
+  g_assert (PEAS_IS_ENGINE (engine));
+
+  if ((schemas = g_strdup (peas_plugin_info_get_external_data (plugin_info, "Settings-Schemas"))))
+    {
+      g_auto(GStrv) split = g_strsplit (g_strdelimit (schemas, " ,\t:", ';'), ";", 0);
+
+      for (guint i = 0; split[i]; i++)
+        {
+          g_strstrip (split[i]);
+
+          if (!ide_str_empty0 (split[i]))
+            ide_context_unregister_settings (self, split[i]);
+        }
+    }
+}
+
 static void
 ide_context_constructed (GObject *object)
 {
   IdeContext *self = (IdeContext *)object;
+  g_auto(GStrv) loaded_plugins = NULL;
+  PeasEngine *engine;
 
-  g_assert (IDE_IS_OBJECT (object));
-
-  self->addins = peas_extension_set_new (peas_engine_get_default (),
-                                         IDE_TYPE_CONTEXT_ADDIN,
-                                         NULL);
-
-  g_signal_connect (self->addins,
-                    "extension-added",
-                    G_CALLBACK (ide_context_addin_added_cb),
-                    self);
-
-  g_signal_connect (self->addins,
-                    "extension-removed",
-                    G_CALLBACK (ide_context_addin_removed_cb),
-                    self);
-
-  peas_extension_set_foreach (self->addins,
-                              ide_context_addin_added_cb,
-                              self);
+  IDE_ENTRY;
 
   G_OBJECT_CLASS (ide_context_parent_class)->constructed (object);
+
+  for (guint i = 0; i < G_N_ELEMENTS (app_schema_ids); i++)
+    {
+      g_autoptr(IdeSettings) settings = ide_settings_new (NULL, app_schema_ids[i]);
+      g_autofree char *prefix = g_strconcat ("settings.app:", app_schema_ids[i], NULL);
+
+      ide_action_muxer_insert_action_group (self->action_muxer,
+                                            prefix,
+                                            G_ACTION_GROUP (settings));
+    }
+
+  engine = peas_engine_get_default ();
+  g_signal_connect_object (engine,
+                           "load-plugin",
+                           G_CALLBACK (ide_context_engine_load_plugin_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  g_signal_connect_object (engine,
+                           "unload-plugin",
+                           G_CALLBACK (ide_context_engine_unload_plugin_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  loaded_plugins = peas_engine_get_loaded_plugins (engine);
+
+  for (guint i = 0; loaded_plugins[i]; i++)
+    {
+      const PeasPluginInfo *info = peas_engine_get_plugin_info (engine, loaded_plugins[i]);
+      ide_context_engine_load_plugin_cb (self, info, engine);
+    }
+
+  IDE_EXIT;
 }
 
 static void
@@ -190,7 +205,13 @@ ide_context_destroy (IdeObject *object)
 
   g_assert (IDE_IS_OBJECT (object));
 
-  g_clear_object (&self->addins);
+  if (self->action_muxer)
+    {
+      g_auto(GStrv) groups = ide_action_muxer_list_groups (self->action_muxer);
+
+      for (guint i = 0; groups[i]; i++)
+        ide_action_muxer_remove_action_group (self->action_muxer, groups[i]);
+    }
 
   IDE_OBJECT_CLASS (ide_context_parent_class)->destroy (object);
 }
@@ -200,6 +221,7 @@ ide_context_finalize (GObject *object)
 {
   IdeContext *self = (IdeContext *)object;
 
+  g_clear_object (&self->action_muxer);
   g_clear_object (&self->workdir);
   g_clear_pointer (&self->project_id, g_free);
   g_clear_pointer (&self->title, g_free);
@@ -290,8 +312,6 @@ ide_context_class_init (IdeContextClass *klass)
    *
    * Before any project has loaded, this is "empty" to allow flexibility
    * for non-project use.
-   *
-   * Since: 3.32
    */
   properties [PROP_PROJECT_ID] =
     g_param_spec_string ("project-id",
@@ -304,8 +324,6 @@ ide_context_class_init (IdeContextClass *klass)
    * IdeContext:title:
    *
    * The "title" property is a descriptive name for the project.
-   *
-   * Since: 3.32
    */
   properties [PROP_TITLE] =
     g_param_spec_string ("title",
@@ -320,8 +338,6 @@ ide_context_class_init (IdeContextClass *klass)
    * The "workdir" property is the best guess at the working directory for the
    * context. This may be discovered using a common parent if multiple files
    * are opened without a project.
-   *
-   * Since: 3.32
    */
   properties [PROP_WORKDIR] =
     g_param_spec_object ("workdir",
@@ -340,8 +356,6 @@ ide_context_class_init (IdeContextClass *klass)
    * @message: the log message
    *
    * This signal is emitted when a log item has been added for the context.
-   *
-   * Since: 3.32
    */
   signals [LOG] =
     g_signal_new_class_handler ("log",
@@ -364,6 +378,7 @@ ide_context_init (IdeContext *self)
   self->workdir = g_file_new_for_path (g_get_home_dir ());
   self->project_id = g_strdup ("empty");
   self->title = g_strdup (_("Untitled"));
+  self->action_muxer = ide_action_muxer_new ();
 
   notifs = ide_notifications_new ();
   ide_object_append (IDE_OBJECT (self), IDE_OBJECT (notifs));
@@ -379,8 +394,6 @@ ide_context_init (IdeContext *self)
  * g_async_initable_init_async().
  *
  * Returns: (transfer full): an #IdeContext
- *
- * Since: 3.32
  */
 IdeContext *
 ide_context_new (void)
@@ -423,8 +436,6 @@ ide_context_peek_child_typed_cb (IdeObject *object,
  *
  * Returns: (transfer none) (type IdeObject) (nullable): an #IdeObject that
  *   matches @type if successful; otherwise %NULL
- *
- * Since: 3.32
  */
 gpointer
 ide_context_peek_child_typed (IdeContext *self,
@@ -452,8 +463,6 @@ ide_context_peek_child_typed (IdeContext *self,
  * Copies the project-id and returns it to the caller.
  *
  * Returns: (transfer full): a project-id as a string
- *
- * Since: 3.32
  */
 gchar *
 ide_context_dup_project_id (IdeContext *self)
@@ -478,13 +487,14 @@ ide_context_dup_project_id (IdeContext *self)
  * Sets the project-id for the context.
  *
  * Generally, this should only be done once after loading a project.
- *
- * Since: 3.32
  */
 void
-ide_context_set_project_id (IdeContext  *self,
-                            const gchar *project_id)
+ide_context_set_project_id (IdeContext *self,
+                            const char *project_id)
 {
+  gboolean register_settings = FALSE;
+
+  g_return_if_fail (IDE_IS_MAIN_THREAD ());
   g_return_if_fail (IDE_IS_CONTEXT (self));
 
   if (ide_str_empty0 (project_id))
@@ -496,8 +506,15 @@ ide_context_set_project_id (IdeContext  *self,
       g_free (self->project_id);
       self->project_id = g_strdup (project_id);
       ide_object_notify_by_pspec (IDE_OBJECT (self), properties [PROP_PROJECT_ID]);
+      register_settings = TRUE;
     }
   ide_object_unlock (IDE_OBJECT (self));
+
+  if (register_settings)
+    {
+      for (guint i = 0; i < G_N_ELEMENTS (project_schema_ids); i++)
+        ide_context_register_settings (self, project_schema_ids[i]);
+    }
 }
 
 /**
@@ -508,8 +525,6 @@ ide_context_set_project_id (IdeContext  *self,
  * reference count by one.
  *
  * Returns: (transfer full): a #GFile
- *
- * Since: 3.32
  */
 GFile *
 ide_context_ref_workdir (IdeContext *self)
@@ -536,8 +551,6 @@ ide_context_ref_workdir (IdeContext *self)
  *
  * In future releases, changes may be made to change this in support of
  * git-worktrees or similar workflows.
- *
- * Since: 3.32
  */
 void
 ide_context_set_workdir (IdeContext *self,
@@ -560,8 +573,6 @@ ide_context_set_workdir (IdeContext *self,
  * Like ide_context_cache_filename() but returns a #GFile.
  *
  * Returns: (transfer full): a #GFile for the cache file
- *
- * Since: 3.32
  */
 GFile *
 ide_context_cache_file (IdeContext  *self,
@@ -617,8 +628,6 @@ ide_context_cache_file (IdeContext  *self,
  * of g_get_user_cache_dir().
  *
  * Returns: (transfer full): A new string containing the cache filename
- *
- * Since: 3.32
  */
 gchar *
 ide_context_cache_filename (IdeContext  *self,
@@ -669,8 +678,6 @@ ide_context_cache_filename (IdeContext  *self,
  * - Otherwise, a #GFile child of #IdeContext:workdir is returned.
  *
  * Returns: (transfer full): a #GFile
- *
- * Since: 3.32
  */
 GFile *
 ide_context_build_file (IdeContext  *self,
@@ -701,8 +708,6 @@ ide_context_build_file (IdeContext  *self,
  * loaded project.
  *
  * Returns: (transfer full): a string containing the new path
- *
- * Since: 3.32
  */
 gchar *
 ide_context_build_filename (IdeContext  *self,
@@ -750,8 +755,6 @@ ide_context_build_filename (IdeContext  *self,
  * This creates a new #GSettings instance for the project.
  *
  * Returns: (transfer full): a #GSettings
- *
- * Since: 3.32
  */
 GSettings *
 ide_context_ref_project_settings (IdeContext *self)
@@ -767,13 +770,45 @@ ide_context_ref_project_settings (IdeContext *self)
   return g_settings_new_with_path ("org.gnome.builder.project", path);
 }
 
+/**
+ * ide_context_ref_settings:
+ * @self: a #IdeContext
+ *
+ * Gets an #IdeSettings for @schema_id.
+ *
+ * The #IdeSettings will read settings from project overrides before
+ * falling back to application settings.
+ *
+ * Changes to settings will always apply to the project overrides.
+ *
+ * Returns: (transfer full) (nullable): an #IdeSettings
+ */
+IdeSettings *
+ide_context_ref_settings (IdeContext *self,
+                          const char *schema_id)
+{
+  g_autoptr(IdeActionMuxer) muxer = NULL;
+
+  g_return_val_if_fail (IDE_IS_CONTEXT (self), NULL);
+  g_return_val_if_fail (schema_id != NULL, NULL);
+
+  if ((muxer = ide_context_ref_action_muxer (self)))
+    {
+      g_autofree char *prefix = g_strconcat ("settings.project:", schema_id, NULL);
+      GActionGroup *group = ide_action_muxer_get_action_group (muxer, prefix);
+
+      if (IDE_IS_SETTINGS (group))
+        return g_object_ref (IDE_SETTINGS (group));
+    }
+
+  return NULL;
+}
+
 /**
  * ide_context_dup_title:
  * @self: a #IdeContext
  *
  * Returns: (transfer full): a string containing the title
- *
- * Since: 3.32
  */
 gchar *
 ide_context_dup_title (IdeContext *self)
@@ -797,8 +832,6 @@ ide_context_dup_title (IdeContext *self)
  * Sets the #IdeContext:title property. This is used by various
  * components to show the user the name of the project. This may
  * include the omnibar and the window title.
- *
- * Since: 3.32
  */
 void
 ide_context_set_title (IdeContext  *self,
@@ -837,8 +870,6 @@ ide_context_log (IdeContext     *self,
  * Checks to see if a project has been loaded in @context.
  *
  * Returns: %TRUE if a project has been, or is currently, loading.
- *
- * Since: 3.32
  */
 gboolean
 ide_context_has_project (IdeContext *self)
@@ -865,35 +896,74 @@ _ide_context_set_has_project (IdeContext *self)
 }
 
 /**
- * ide_context_addin_find_by_module_name:
- * @context: an #IdeContext
- * @module_name: the name of the addin module
+ * ide_context_ref_action_muxer:
+ * @self: a #IdeContext
  *
- * Finds the addin (if any) matching the plugin's @module_name.
+ * Gets the action muxer for the context.
  *
- * Returns: (transfer none) (nullable): an #IdeContextAddin or %NULL
- *
- * Since: 3.40
+ * Returns: (transfer full): an #IdeActionMuxer
  */
-IdeContextAddin *
-ide_context_addin_find_by_module_name (IdeContext  *context,
-                                       const gchar *module_name)
+IdeActionMuxer *
+ide_context_ref_action_muxer (IdeContext *self)
 {
-  PeasPluginInfo *plugin_info;
-  PeasExtension *ret = NULL;
-  PeasEngine *engine;
+  IdeActionMuxer *ret = NULL;
 
-  g_return_val_if_fail (IDE_IS_MAIN_THREAD (), NULL);
-  g_return_val_if_fail (IDE_IS_CONTEXT (context), NULL);
-  g_return_val_if_fail (module_name != NULL, NULL);
+  g_return_val_if_fail (IDE_IS_CONTEXT (self), NULL);
 
-  if (context->addins == NULL)
-    return NULL;
+  ide_object_lock (IDE_OBJECT (self));
+  g_set_object (&ret, self->action_muxer);
+  ide_object_unlock (IDE_OBJECT (self));
 
-  engine = peas_engine_get_default ();
+  return g_steal_pointer (&ret);
+}
 
-  if ((plugin_info = peas_engine_get_plugin_info (engine, module_name)))
-    ret = peas_extension_set_get_extension (context->addins, plugin_info);
+/**
+ * ide_context_register_settings:
+ * @self: a #IdeContext
+ *
+ * Registers settings that can be overriden by a project.
+ *
+ * Use this if you have a GSettings schema that can have both
+ * application and per-project overrides.
+ */
+void
+ide_context_register_settings (IdeContext *self,
+                               const char *schema_id)
+{
+  g_autoptr(IdeActionMuxer) muxer = NULL;
+  g_autofree char *project_id = NULL;
 
-  return IDE_CONTEXT_ADDIN (ret);
+  g_return_if_fail (IDE_IS_CONTEXT (self));
+  g_return_if_fail (schema_id != NULL);
+
+  if ((muxer = ide_context_ref_action_muxer (self)) &&
+      (project_id = ide_context_dup_project_id (self)))
+    {
+      g_autoptr(IdeSettings) project_settings = ide_settings_new (project_id, schema_id);
+      g_autoptr(IdeSettings) app_settings = ide_settings_new (NULL, schema_id);
+      g_autofree char *project_group = g_strconcat ("settings.project:", schema_id, NULL);
+      g_autofree char *app_group = g_strconcat ("settings.app:", schema_id, NULL);
+
+      ide_action_muxer_insert_action_group (muxer, app_group, G_ACTION_GROUP (app_settings));
+      ide_action_muxer_insert_action_group (muxer, project_group, G_ACTION_GROUP (project_settings));
+    }
+}
+
+void
+ide_context_unregister_settings (IdeContext *self,
+                                 const char *schema_id)
+{
+  g_autoptr(IdeActionMuxer) muxer = NULL;
+
+  g_return_if_fail (IDE_IS_CONTEXT (self));
+  g_return_if_fail (schema_id != NULL);
+
+  if ((muxer = ide_context_ref_action_muxer (self)))
+    {
+      g_autofree char *project_group = g_strconcat ("settings.project:", schema_id, NULL);
+      g_autofree char *app_group = g_strconcat ("settings.app:", schema_id, NULL);
+
+      ide_action_muxer_insert_action_group (muxer, project_group, NULL);
+      ide_action_muxer_insert_action_group (muxer, app_group, NULL);
+    }
 }
