@@ -24,17 +24,23 @@
 
 #define IDE_CORE_INSIDE
 
+#include "ide-action-group.h"
+#include "ide-action-muxer.h"
+#include "ide-binding-group.h"
 #include "ide-context.h"
-#include "ide-context-addin.h"
 #include "ide-debug.h"
 #include "ide-global.h"
+#include "ide-gsettings-action-group.h"
 #include "ide-log.h"
 #include "ide-macros.h"
 #include "ide-notification.h"
 #include "ide-notifications.h"
 #include "ide-object.h"
 #include "ide-object-box.h"
+#include "ide-property-action-group.h"
 #include "ide-settings.h"
+#include "ide-settings-flag-action.h"
+#include "ide-signal-group.h"
 #include "ide-transfer.h"
 #include "ide-transfer-manager.h"
 #include "ide-version.h"
