@@ -37,6 +37,7 @@
 #include "ide-config.h"
 #include "ide-device.h"
 #include "ide-foundry-compat.h"
+#include "ide-runtime.h"
 #include "ide-toolchain.h"
 
 G_DEFINE_INTERFACE (IdeBuildSystem, ide_build_system, IDE_TYPE_OBJECT)
@@ -317,35 +318,36 @@ ide_build_system_default_init (IdeBuildSystemInterface *iface)
   g_object_interface_install_property (iface, properties [PROP_PROJECT_FILE]);
 }
 
-static gchar *
-ide_build_system_translate (IdeBuildSystem   *self,
-                            IdePipeline *pipeline,
-                            const gchar      *prefix,
-                            const gchar      *path)
+static char *
+ide_build_system_translate (IdeBuildSystem *self,
+                            IdePipeline    *pipeline,
+                            const char     *prefix,
+                            const char     *path)
 {
-  g_autofree gchar *freeme = NULL;
-  g_autofree gchar *translated_path = NULL;
-  g_autoptr(GFile) file = NULL;
-  g_autoptr(GFile) translated = NULL;
-  IdeRuntime *runtime;
+  IdeConfig *config;
 
   g_assert (IDE_IS_BUILD_SYSTEM (self));
   g_assert (!pipeline || IDE_IS_PIPELINE (pipeline));
   g_assert (prefix != NULL);
   g_assert (path != NULL);
 
-  if (NULL == pipeline ||
-      NULL == (runtime = ide_pipeline_get_runtime (pipeline)))
-    return g_strdup_printf ("%s%s", prefix, path);
+  if (pipeline && (config = ide_pipeline_get_config (pipeline)))
+    {
+      g_autofree char *freeme = NULL;
+      g_autoptr(GFile) file = NULL;
+      g_autoptr(GFile) translated = NULL;
 
-  if (!g_path_is_absolute (path))
-    path = freeme = ide_pipeline_build_builddir_path (pipeline, path, NULL);
+      if (!g_path_is_absolute (path))
+        path = freeme = ide_pipeline_build_builddir_path (pipeline, path, NULL);
 
-  file = g_file_new_for_path (path);
-  translated = ide_runtime_translate_file (runtime, file);
-  translated_path = g_file_get_path (translated);
+      file = g_file_new_for_path (path);
 
-  return g_strdup_printf ("%s%s", prefix, translated_path);
+      if ((translated = ide_config_translate_file (config, file)) &&
+          g_file_is_native (translated))
+        return g_strdup_printf ("%s%s", prefix, g_file_peek_path (translated));
+    }
+
+  return g_strdup_printf ("%s%s", prefix, path);
 }
 
 static void
@@ -367,6 +369,7 @@ ide_build_system_post_process_build_flags (IdeBuildSystem  *self,
 
   for (guint i = 0; flags[i] != NULL; i++)
     {
+      const char *next = flags[i+1];
       gchar *flag = flags[i];
       gchar *translated;
 
@@ -402,7 +405,15 @@ ide_build_system_post_process_build_flags (IdeBuildSystem  *self,
         case 'f': /* -fPIC */
         case 'W': /* -Werror... */
         case 'm': /* -m64 -mtune=native */
+
         default:
+          if (next != NULL &&
+              (ide_str_equal0 (flag, "-include") ||
+               ide_str_equal0 (flag, "-isystem")))
+            {
+              translated = ide_build_system_translate (self, pipeline, "", next);
+              ide_take_string (&flags[i+1], translated);
+            }
           break;
         }
     }
@@ -430,8 +441,6 @@ ide_build_system_get_build_flags_async (IdeBuildSystem      *self,
  * ide_build_system_get_build_flags_finish:
  *
  * Returns: (transfer full):
- *
- * Since: 3.32
  */
 gchar **
 ide_build_system_get_build_flags_finish (IdeBuildSystem  *self,
@@ -476,8 +485,6 @@ ide_build_system_get_build_flags_finish (IdeBuildSystem  *self,
  *
  * This function will get build flags for all files and returns
  * map of file and its build flags as #GHashTable.
- *
- * Since: 3.32
  */
 void
 ide_build_system_get_build_flags_for_files_async (IdeBuildSystem       *self,
@@ -504,8 +511,6 @@ ide_build_system_get_build_flags_for_files_async (IdeBuildSystem       *self,
  * @error: a location for a #GError or %NULL
  *
  * Returns: (element-type Ide.File GStrv) (transfer full): a #GHashTable or #GFile to #GStrv
- *
- * Since: 3.32
  */
 GHashTable *
 ide_build_system_get_build_flags_for_files_finish (IdeBuildSystem  *self,
@@ -727,8 +732,6 @@ ide_build_system_get_build_flags_for_dir_async (IdeBuildSystem      *self,
  * @error: a location for a #GError or %NULL
  *
  * Returns: (element-type Ide.File GStrv) (transfer full): a #GHashTable of #GFile to #GStrv
- *
- * Since: 3.32
  */
 GHashTable *
 ide_build_system_get_build_flags_for_dir_finish (IdeBuildSystem  *self,
@@ -749,8 +752,6 @@ ide_build_system_get_build_flags_for_dir_finish (IdeBuildSystem  *self,
  * Checks whether the build system supports the given toolchain.
  *
  * Returns: %TRUE if the toolchain is supported by the build system, %FALSE otherwise
- *
- * Since: 3.32
  */
 gboolean
 ide_build_system_supports_toolchain (IdeBuildSystem *self,
@@ -779,8 +780,6 @@ ide_build_system_supports_toolchain (IdeBuildSystem *self,
  * in the build system's configuration files.
  *
  * Returns: (transfer full) (nullable): a string containing the project version
- *
- * Since: 3.32
  */
 gchar *
 ide_build_system_get_project_version (IdeBuildSystem *self)
@@ -802,8 +801,6 @@ ide_build_system_get_project_version (IdeBuildSystem *self)
  * Returns %TRUE if @self in it's current configuration is known to support @language.
  *
  * Returns: %TRUE if @language is supported, otherwise %FALSE.
- *
- * Since: 41.0
  */
 gboolean
 ide_build_system_supports_language (IdeBuildSystem *self,
