@@ -24,6 +24,8 @@
 
 #include <string.h>
 
+#include <gtk/gtk.h>
+
 #include "ide-config-manager.h"
 #include "ide-config-private.h"
 #include "ide-config.h"
@@ -103,6 +105,8 @@ enum {
   PROP_TOOLCHAIN_ID,
   PROP_TOOLCHAIN,
   PROP_RUN_OPTS,
+  PROP_SUPPORTED_RUNTIMES,
+  PROP_DESCRIPTION,
   N_PROPS
 };
 
@@ -168,6 +172,21 @@ ide_config_emit_changed (IdeConfig *self)
     g_signal_emit (self, signals [CHANGED], 0);
 }
 
+static GFile *
+ide_config_real_translate_file (IdeConfig *self,
+                                GFile     *file)
+{
+  IdeRuntime *runtime;
+
+  g_assert (IDE_IS_CONFIG (self));
+  g_assert (G_IS_FILE (file));
+
+  if ((runtime = ide_config_get_runtime (self)))
+    return ide_runtime_translate_file (runtime, file);
+
+  return g_object_ref (file);
+}
+
 static IdeRuntime *
 ide_config_real_get_runtime (IdeConfig *self)
 {
@@ -289,6 +308,36 @@ ide_config_real_set_runtime (IdeConfig  *self,
   ide_config_set_runtime_id (self, runtime_id);
 }
 
+static gboolean
+filter_supported_runtime_cb (gpointer item,
+                             gpointer user_data)
+{
+  return ide_config_supports_runtime (user_data, item);
+}
+
+static GListModel *
+ide_config_get_supported_runtimes (IdeConfig *self)
+{
+  IdeContext *context = ide_object_get_context (IDE_OBJECT (self));
+  GListModel *runtimes = G_LIST_MODEL (ide_runtime_manager_from_context (context));
+  GtkCustomFilter *filter = gtk_custom_filter_new (filter_supported_runtime_cb, g_object_ref (self), g_object_unref);
+
+  return G_LIST_MODEL (gtk_filter_list_model_new (g_object_ref (runtimes), GTK_FILTER (filter)));
+}
+
+static void
+ide_config_set_environ (IdeConfig          *self,
+                        const char * const *environ)
+{
+  IdeEnvironment *env;
+
+  g_assert (IDE_IS_CONFIG (self));
+
+  env = ide_config_get_environment (self);
+  ide_environment_set_environ (env, environ);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_ENVIRON]);
+}
+
 static gchar *
 ide_config_repr (IdeObject *object)
 {
@@ -323,6 +372,7 @@ ide_config_finalize (GObject *object)
   g_clear_pointer (&priv->id, g_free);
   g_clear_pointer (&priv->post_install_commands, g_strfreev);
   g_clear_pointer (&priv->prefix, g_free);
+  g_clear_pointer (&priv->run_opts, g_free);
   g_clear_pointer (&priv->runtime_id, g_free);
   g_clear_pointer (&priv->app_id, g_free);
   g_clear_pointer (&priv->toolchain_id, g_free);
@@ -342,6 +392,10 @@ ide_config_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_DESCRIPTION:
+      g_value_take_string (value, ide_config_get_description (self));
+      break;
+
     case PROP_CONFIG_OPTS:
       g_value_set_string (value, ide_config_get_config_opts (self));
       break;
@@ -426,6 +480,10 @@ ide_config_get_property (GObject    *object,
       g_value_set_flags (value, ide_config_get_locality (self));
       break;
 
+    case PROP_SUPPORTED_RUNTIMES:
+      g_value_take_object (value, ide_config_get_supported_runtimes (self));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -465,6 +523,10 @@ ide_config_set_property (GObject      *object,
       ide_config_set_display_name (self, g_value_get_string (value));
       break;
 
+    case PROP_ENVIRON:
+      ide_config_set_environ (self, g_value_get_boxed (value));
+      break;
+
     case PROP_ID:
       ide_config_set_id (self, g_value_get_string (value));
       break;
@@ -477,6 +539,10 @@ ide_config_set_property (GObject      *object,
       ide_config_set_prefix (self, g_value_get_string (value));
       break;
 
+    case PROP_PREFIX_SET:
+      ide_config_set_prefix_set (self, g_value_get_boolean (value));
+      break;
+
     case PROP_PARALLELISM:
       ide_config_set_parallelism (self, g_value_get_int (value));
       break;
@@ -536,6 +602,12 @@ ide_config_class_init (IdeConfigClass *klass)
 
   klass->get_runtime = ide_config_real_get_runtime;
   klass->set_runtime = ide_config_real_set_runtime;
+  klass->translate_file = ide_config_real_translate_file;
+
+  properties[PROP_DESCRIPTION] =
+    g_param_spec_string ("description", NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   properties [PROP_PREPEND_PATH] =
     g_param_spec_string ("prepend-path",
@@ -598,7 +670,7 @@ ide_config_class_init (IdeConfigClass *klass)
                         "Environ",
                         "Environ",
                         G_TYPE_STRV,
-                        (G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
+                        (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
   properties [PROP_ID] =
     g_param_spec_string ("id",
@@ -694,6 +766,11 @@ ide_config_class_init (IdeConfigClass *klass)
                         IDE_BUILD_LOCALITY_DEFAULT,
                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  properties[PROP_SUPPORTED_RUNTIMES] =
+    g_param_spec_object ("supported-runtimes", NULL, NULL,
+                         G_TYPE_LIST_MODEL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_properties (object_class, N_PROPS, properties);
 
   signals [CHANGED] =
@@ -729,8 +806,6 @@ ide_config_init (IdeConfig *self)
  * Gets the application ID for the configuration.
  *
  * Returns: (transfer none) (nullable): A string.
- *
- * Since: 3.32
  */
 const gchar *
 ide_config_get_app_id (IdeConfig *self)
@@ -813,8 +888,6 @@ ide_config_set_runtime_id (IdeConfig   *self,
  * Gets the toolchain id for the configuration.
  *
  * Returns: (transfer none) (nullable): The id of an #IdeToolchain or %NULL
- *
- * Since: 3.32
  */
 const gchar *
 ide_config_get_toolchain_id (IdeConfig *self)
@@ -832,8 +905,6 @@ ide_config_get_toolchain_id (IdeConfig *self)
  * @toolchain_id: The id of an #IdeToolchain
  *
  * Sets the toolchain id for the configuration.
- *
- * Since: 3.32
  */
 void
 ide_config_set_toolchain_id (IdeConfig   *self,
@@ -866,8 +937,6 @@ ide_config_set_toolchain_id (IdeConfig   *self,
  * Gets the runtime for the configuration.
  *
  * Returns: (transfer none) (nullable): An #IdeRuntime
- *
- * Since: 3.32
  */
 IdeRuntime *
 ide_config_get_runtime (IdeConfig *self)
@@ -894,8 +963,6 @@ ide_config_set_runtime (IdeConfig  *self,
  * Gets the toolchain for the configuration.
  *
  * Returns: (transfer full) (nullable): An #IdeToolchain
- *
- * Since: 3.32
  */
 IdeToolchain *
 ide_config_get_toolchain (IdeConfig *self)
@@ -923,8 +990,6 @@ ide_config_get_toolchain (IdeConfig *self)
  * @toolchain: (nullable): An #IdeToolchain or %NULL to use the default one
  *
  * Sets the toolchain for the configuration.
- *
- * Since: 3.32
  */
 void
 ide_config_set_toolchain (IdeConfig    *self,
@@ -948,8 +1013,6 @@ ide_config_set_toolchain (IdeConfig    *self,
  * Gets the environment to use when spawning processes.
  *
  * Returns: (transfer full): An array of key=value environment variables.
- *
- * Since: 3.32
  */
 gchar **
 ide_config_get_environ (IdeConfig *self)
@@ -1163,8 +1226,6 @@ ide_config_set_dirty (IdeConfig *self,
  * ide_config_get_environment:
  *
  * Returns: (transfer none): An #IdeEnvironment.
- *
- * Since: 3.32
  */
 IdeEnvironment *
 ide_config_get_environment (IdeConfig *self)
@@ -1213,8 +1274,6 @@ ide_config_set_environment (IdeConfig      *self,
  * ide_config_get_runtime_environment:
  *
  * Returns: (transfer none): An #IdeEnvironment.
- *
- * Since: 3.40
  */
 IdeEnvironment *
 ide_config_get_runtime_environment (IdeConfig *self)
@@ -1349,8 +1408,6 @@ ide_config_set_post_install_commands (IdeConfig           *self,
  * mean time.
  *
  * Returns: A monotonic sequence number.
- *
- * Since: 3.32
  */
 guint
 ide_config_get_sequence (IdeConfig *self)
@@ -1558,8 +1615,6 @@ ide_config_set_internal_int64 (IdeConfig   *self,
  * Gets the value associated with @key if it is a #GObject.
  *
  * Returns: (nullable) (transfer none) (type GObject.Object): a #GObject or %NULL.
- *
- * Since: 3.32
  */
 gpointer
 ide_config_get_internal_object (IdeConfig   *self,
@@ -1586,8 +1641,6 @@ ide_config_get_internal_object (IdeConfig   *self,
  * @instance: (type GObject.Object) (nullable): a #GObject or %NULL
  *
  * Sets the value for @key to @instance.
- *
- * Since: 3.32
  */
 void
 ide_config_set_internal_object (IdeConfig   *self,
@@ -1616,8 +1669,6 @@ ide_config_set_internal_object (IdeConfig   *self,
  * Determines if the configuration is ready for use.
  *
  * Returns: %TRUE if the configuration is ready for use.
- *
- * Since: 3.32
  */
 gboolean
 ide_config_get_ready (IdeConfig *self)
@@ -1656,8 +1707,6 @@ ide_config_supports_runtime (IdeConfig  *self,
  *
  * Returns: (transfer none) (nullable): A string containing the run options
  *   or %NULL if none have been set.
- *
- * Since: 3.32
  */
 const gchar *
 ide_config_get_run_opts (IdeConfig *self)
@@ -1676,8 +1725,6 @@ ide_config_get_run_opts (IdeConfig *self)
  *
  * Sets the run options to use when running the target application.
  * See ide_config_get_run_opts() for more information.
- *
- * Since: 3.32
  */
 void
 ide_config_set_run_opts (IdeConfig   *self,
@@ -1692,6 +1739,7 @@ ide_config_set_run_opts (IdeConfig   *self,
       g_free (priv->run_opts);
       priv->run_opts = g_strdup (run_opts);
       g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_RUN_OPTS]);
+      ide_config_set_dirty (self, TRUE);
     }
 }
 
@@ -1795,8 +1843,6 @@ ide_config_set_locality (IdeConfig        *self,
  * @self: a #IdeConfig
  *
  * Returns: (transfer none) (nullable): a #GFile or %NULL
- *
- * Since: 3.32
  */
 GFile *
 ide_config_get_build_commands_dir (IdeConfig *self)
@@ -1897,8 +1943,6 @@ ide_config_set_prefix_set (IdeConfig *self,
  *
  * Returns: (not nullable) (transfer full) (element-type Ide.Runtime): an array
  *   of #IdeRuntime for the runtime extensions for the configuration.
- *
- * Since: 3.34
  */
 GPtrArray *
 ide_config_get_extensions (IdeConfig *self)
@@ -1940,3 +1984,50 @@ ide_config_set_args_for_phase (IdeConfig           *self,
 
   g_hash_table_insert (priv->pipeline_args, GINT_TO_POINTER (phase), g_strdupv ((gchar **)args));
 }
+
+/**
+ * ide_config_get_description:
+ * @self: a #IdeConfig
+ *
+ * Describes the type of config this is.
+ *
+ * Examples might include ".buildconfig" or "Flatpak".
+ *
+ * Returns: (transfer full): a string describing the configuration.
+ */
+char *
+ide_config_get_description (IdeConfig *self)
+{
+  char *ret = NULL;
+
+  g_return_val_if_fail (IDE_IS_CONFIG (self), NULL);
+
+  if (IDE_CONFIG_GET_CLASS (self)->get_description)
+    ret = IDE_CONFIG_GET_CLASS (self)->get_description (self);
+
+  if (ret == NULL)
+    ret = g_strdup (G_OBJECT_TYPE_NAME (self));
+
+  return ret;
+}
+
+/**
+ * ide_config_translate_file:
+ * @self: a #IdeConfig
+ *
+ * Requests translation of the file path to one available in the
+ * current process. That might mean translating to a path that
+ * allows access outside Builder's sandbox such as using
+ * /var/run/host or depoy-directories of OSTree commits.
+ *
+ * Returns: (transfer full) (nullable): a #GFile or %NULL
+ */
+GFile *
+ide_config_translate_file (IdeConfig *self,
+                           GFile     *file)
+{
+  g_return_val_if_fail (IDE_IS_CONFIG (self), NULL);
+  g_return_val_if_fail (G_IS_FILE (file), NULL);
+
+  return IDE_CONFIG_GET_CLASS (self)->translate_file (self, file);
+}
