@@ -372,8 +372,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    *
    * This property is set to %TRUE when the build stage is actively
    * running or cleaning.
-   *
-   * Since: 3.32
    */
   properties [PROP_ACTIVE] =
     g_param_spec_boolean ("active",
@@ -391,8 +389,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    * that Builder will extract errors from stdout.
    *
    * One such example is Ninja.
-   *
-   * Since: 3.32
    */
   properties [PROP_CHECK_STDOUT] =
     g_param_spec_boolean ("check-stdout",
@@ -407,8 +403,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    * The "completed" property is set to %TRUE after the pipeline has
    * completed processing the stage. When the pipeline invalidates
    * phases, completed may be reset to %FALSE.
-   *
-   * Since: 3.32
    */
   properties [PROP_COMPLETED] =
     g_param_spec_boolean ("completed",
@@ -427,8 +421,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    *
    * If the stage is both transient and disabled, it will not be removed during
    * the transient cleanup phase.
-   *
-   * Since: 3.32
    */
   properties [PROP_DISABLED] =
     g_param_spec_boolean ("disabled",
@@ -442,8 +434,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    *
    * The name of the build stage. This is only used by UI to view
    * the build pipeline.
-   *
-   * Since: 3.32
    */
   properties [PROP_NAME] =
     g_param_spec_string ("name",
@@ -462,8 +452,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    * For safety reasons, the contents are first redirected to a temporary
    * file and will be redirected to the stdout-path location after the
    * build stage has completed executing.
-   *
-   * Since: 3.32
    */
   properties [PROP_STDOUT_PATH] =
     g_param_spec_string ("stdout-path",
@@ -481,8 +469,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    * ide_pipeline_build_async(). This can be a convenient
    * way to add a temporary item to a build pipeline that should
    * be immediately discarded.
-   *
-   * Since: 3.32
    */
   properties [PROP_TRANSIENT] =
     g_param_spec_boolean ("transient",
@@ -503,8 +489,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    *
    * Returns: %TRUE if @next's work was chained into @self for the next
    *    execution of the pipeline.
-   *
-   * Since: 3.32
    */
   signals [CHAIN] =
     g_signal_new ("chain",
@@ -539,8 +523,6 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
    * and perform an external operation. Forward progress of the stage will
    * be paused until a matching number of ide_pipeline_stage_unpause() calls
    * have been made.
-   *
-   * Since: 3.32
    */
   signals [QUERY] =
     g_signal_new ("query",
@@ -557,15 +539,13 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
   /**
    * IdePipelineStage::reap:
    * @self: An #IdePipelineStage
-   * @reaper: An #DzlDirectoryReaper
+   * @reaper: An #IdeDirectoryReaper
    *
    * This signal is emitted when a request to rebuild the project has
    * occurred. This allows build stages to ensure that certain files are
    * removed from the system. For example, an autotools build stage might
    * request that "configure" is removed so that autogen.sh will be Executed
    * as part of the next build.
-   *
-   * Since: 3.32
    */
   signals [REAP] =
     g_signal_new ("reap",
@@ -573,7 +553,7 @@ ide_pipeline_stage_class_init (IdePipelineStageClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (IdePipelineStageClass, reap),
                   NULL, NULL, NULL,
-                  G_TYPE_NONE, 1, DZL_TYPE_DIRECTORY_REAPER);
+                  G_TYPE_NONE, 1, IDE_TYPE_DIRECTORY_REAPER);
 }
 
 static void
@@ -660,8 +640,6 @@ ide_pipeline_stage_build_finish (IdePipelineStage  *self,
  * Plugins that need to handle logging from a build stage should set
  * an observer on the pipeline so that log distribution may be fanned
  * out to all observers.
- *
- * Since: 3.32
  */
 void
 ide_pipeline_stage_set_log_observer (IdePipelineStage    *self,
@@ -864,8 +842,6 @@ ide_pipeline_stage_observe_stream (IdePipelineStage  *self,
  * stdout and stderr streams of the subprocess. You must have created
  * the subprocess with %G_SUBPROCESS_FLAGS_STDERR_PIPE and
  * %G_SUBPROCESS_FLAGS_STDOUT_PIPE so that the streams may be read.
- *
- * Since: 3.32
  */
 void
 ide_pipeline_stage_log_subprocess (IdePipelineStage *self,
@@ -958,8 +934,6 @@ ide_pipeline_stage_unpause (IdePipelineStage *self)
  * query signal. If the stage is paused after the query, build will
  * be delayed until the correct number of ide_pipeline_stage_unpause() calls
  * have occurred.
- *
- * Since: 3.32
  */
 void
 _ide_pipeline_stage_build_with_query_async (IdePipelineStage    *self,
@@ -1085,12 +1059,12 @@ ide_pipeline_stage_clean_finish (IdePipelineStage  *self,
 
 void
 ide_pipeline_stage_emit_reap (IdePipelineStage   *self,
-                              DzlDirectoryReaper *reaper)
+                              IdeDirectoryReaper *reaper)
 {
   IDE_ENTRY;
 
   g_return_if_fail (IDE_IS_PIPELINE_STAGE (self));
-  g_return_if_fail (DZL_IS_DIRECTORY_REAPER (reaper));
+  g_return_if_fail (IDE_IS_DIRECTORY_REAPER (reaper));
 
   g_signal_emit (self, signals [REAP], 0, reaper);
 
@@ -1176,8 +1150,6 @@ ide_pipeline_stage_set_check_stdout (IdePipelineStage *self,
  * build stage is actively executing or cleaning.
  *
  * Returns: %TRUE if the stage is actively executing or cleaning.
- *
- * Since: 3.32
  */
 gboolean
 ide_pipeline_stage_get_active (IdePipelineStage *self)
