@@ -22,7 +22,6 @@
 
 #include "config.h"
 
-#include <dazzle.h>
 #include <glib/gi18n.h>
 
 #include "ide-extension-adapter.h"
@@ -36,7 +35,7 @@ struct _IdeExtensionAdapter
   gchar          *key;
   gchar          *value;
   GObject        *extension;
-  DzlSignalGroup *settings_signals;
+  IdeSignalGroup *settings_signals;
   GSettings      *settings;
 
   PeasPluginInfo *plugin_info;
@@ -96,13 +95,13 @@ ide_extension_adapter_monitor (IdeExtensionAdapter *self,
   g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (IDE_IS_EXTENSION_ADAPTER (self));
 
-  dzl_signal_group_set_target (self->settings_signals, NULL);
+  ide_signal_group_set_target (self->settings_signals, NULL);
   g_clear_object (&self->settings);
 
   if (plugin_info != NULL)
     {
       self->settings = ide_extension_adapter_get_settings (self, plugin_info);
-      dzl_signal_group_set_target (self->settings_signals, self->settings);
+      ide_signal_group_set_target (self->settings_signals, self->settings);
     }
 }
 
@@ -470,8 +469,8 @@ ide_extension_adapter_init (IdeExtensionAdapter *self)
 {
   self->interface_type = G_TYPE_INVALID;
 
-  self->settings_signals = dzl_signal_group_new (G_TYPE_SETTINGS);
-  dzl_signal_group_connect_object (self->settings_signals,
+  self->settings_signals = ide_signal_group_new (G_TYPE_SETTINGS);
+  ide_signal_group_connect_object (self->settings_signals,
                                    "changed::disabled",
                                    G_CALLBACK (ide_extension_adapter__changed_disabled),
                                    self,
@@ -544,8 +543,6 @@ ide_extension_adapter_get_interface_type (IdeExtensionAdapter *self)
  * Gets the #IdeExtensionAdapter:engine property.
  *
  * Returns: (transfer none): a #PeasEngine.
- *
- * Since: 3.32
  */
 PeasEngine *
 ide_extension_adapter_get_engine (IdeExtensionAdapter *self)
@@ -562,8 +559,6 @@ ide_extension_adapter_get_engine (IdeExtensionAdapter *self)
  * Gets the extension object managed by the adapter.
  *
  * Returns: (transfer none) (type GObject.Object): a #GObject or %NULL.
- *
- * Since: 3.32
  */
 gpointer
 ide_extension_adapter_get_extension (IdeExtensionAdapter *self)
@@ -598,8 +593,6 @@ ide_extension_adapter_get_extension (IdeExtensionAdapter *self)
  * be used.
  *
  * Returns: (transfer full): A newly created #IdeExtensionAdapter.
- *
- * Since: 3.32
  */
 IdeExtensionAdapter *
 ide_extension_adapter_new (IdeObject   *parent,
