@@ -41,8 +41,6 @@
  *
  * This class contains information about a project that can be loaded.
  * This information should be used to display a list of available projects.
- *
- * Since: 3.32
  */
 
 struct _IdeProjectInfo
@@ -69,7 +67,7 @@ struct _IdeProjectInfo
   guint       is_recent : 1;
 };
 
-G_DEFINE_TYPE (IdeProjectInfo, ide_project_info, G_TYPE_OBJECT)
+G_DEFINE_FINAL_TYPE (IdeProjectInfo, ide_project_info, G_TYPE_OBJECT)
 
 enum {
   PROP_0,
@@ -98,8 +96,6 @@ static GParamSpec *properties [LAST_PROP];
  *
  *
  * Returns: (nullable) (transfer none): An #IdeDoap or %NULL.
- *
- * Since: 3.32
  */
 IdeDoap *
 ide_project_info_get_doap (IdeProjectInfo *self)
@@ -124,8 +120,6 @@ ide_project_info_set_doap (IdeProjectInfo *self,
  * ide_project_info_get_languages:
  *
  * Returns: (transfer none): An array of language names.
- *
- * Since: 3.32
  */
 const gchar * const *
 ide_project_info_get_languages (IdeProjectInfo *self)
@@ -178,8 +172,6 @@ ide_project_info_set_priority (IdeProjectInfo *self,
  * This is the directory containing the project (if known).
  *
  * Returns: (nullable) (transfer none): a #GFile.
- *
- * Since: 3.32
  */
 GFile *
 ide_project_info_get_directory (IdeProjectInfo *self)
@@ -197,8 +189,6 @@ ide_project_info_get_directory (IdeProjectInfo *self)
  * This is the project file (such as configure.ac) of the project.
  *
  * Returns: (nullable) (transfer none): a #GFile.
- *
- * Since: 3.32
  */
 GFile *
 ide_project_info_get_file (IdeProjectInfo *self)
@@ -213,8 +203,6 @@ ide_project_info_get_file (IdeProjectInfo *self)
  *
  *
  * Returns: (transfer none) (nullable): a #GDateTime or %NULL.
- *
- * Since: 3.32
  */
 GDateTime *
 ide_project_info_get_last_modified_at (IdeProjectInfo *self)
@@ -723,8 +711,6 @@ ide_project_info_compare (IdeProjectInfo *info1,
  * project when activated from the greeter.
  *
  * Returns: (transfer none) (nullable): a #IdeVcsUri or %NULL
- *
- * Since: 3.32
  */
 const gchar *
 ide_project_info_get_vcs_uri (IdeProjectInfo *self)
@@ -770,8 +756,6 @@ ide_project_info_new (void)
  *
  * Returns: %TRUE if @self and @other are the same project and similar
  *   enough to be considered equal.
- *
- * Since: 3.32
  */
 gboolean
 ide_project_info_equal (IdeProjectInfo *self,
@@ -860,8 +844,6 @@ ide_project_info_set_id (IdeProjectInfo *self,
  * Gets the #IdeProjectInfo:icon property.
  *
  * Returns: (transfer none) (nullable): a #GIcon or %NULL
- *
- * Since: 3.32
  */
 GIcon *
 ide_project_info_get_icon (IdeProjectInfo *self)
