@@ -48,13 +48,17 @@ ide_search_provider_real_search_async (IdeSearchProvider   *self,
                              "search not implemented");
 }
 
-static GPtrArray *
+static GListModel *
 ide_search_provider_real_search_finish (IdeSearchProvider  *self,
                                         GAsyncResult       *result,
+                                        gboolean           *truncated,
                                         GError            **error)
 {
   g_assert (IDE_IS_SEARCH_PROVIDER (self));
   g_assert (IDE_IS_TASK (result));
+  g_assert (truncated != NULL);
+
+  *truncated = FALSE;
 
   return ide_task_propagate_pointer (IDE_TASK (result), error);
 }
@@ -66,6 +70,26 @@ ide_search_provider_default_init (IdeSearchProviderInterface *iface)
   iface->search_finish = ide_search_provider_real_search_finish;
 }
 
+void
+ide_search_provider_load (IdeSearchProvider *self)
+{
+  g_return_if_fail (IDE_IS_MAIN_THREAD ());
+  g_return_if_fail (IDE_IS_SEARCH_PROVIDER (self));
+
+  if (IDE_SEARCH_PROVIDER_GET_IFACE (self)->load)
+    IDE_SEARCH_PROVIDER_GET_IFACE (self)->load (self);
+}
+
+void
+ide_search_provider_unload (IdeSearchProvider *self)
+{
+  g_return_if_fail (IDE_IS_MAIN_THREAD ());
+  g_return_if_fail (IDE_IS_SEARCH_PROVIDER (self));
+
+  if (IDE_SEARCH_PROVIDER_GET_IFACE (self)->unload)
+    IDE_SEARCH_PROVIDER_GET_IFACE (self)->unload (self);
+}
+
 void
 ide_search_provider_search_async (IdeSearchProvider   *self,
                                   const gchar         *query,
@@ -74,33 +98,56 @@ ide_search_provider_search_async (IdeSearchProvider   *self,
                                   GAsyncReadyCallback  callback,
                                   gpointer             user_data)
 {
+  IDE_ENTRY;
+
+  g_return_if_fail (IDE_IS_MAIN_THREAD ());
   g_return_if_fail (IDE_IS_SEARCH_PROVIDER (self));
   g_return_if_fail (query != NULL);
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
   IDE_SEARCH_PROVIDER_GET_IFACE (self)->search_async (self, query, max_results, cancellable, callback, user_data);
+
+  IDE_EXIT;
 }
 
 /**
  * ide_search_provider_search_finish:
  * @self: a #IdeSearchProvider
  * @result: a #GAsyncResult
+ * @truncated: (nullable) (out): if the result was truncated
  * @error: a location for a #GError, or %NULL
  *
  * Completes a request to a search provider.
  *
- * Returns: (transfer full) (element-type IdeSearchResult): a #GPtrArray
- *    of #IdeSearchResult elements.
+ * If the result was truncated because of too many search results, then
+ * @truncated is set to %TRUE.
  *
- * Since: 3.32
+ * Returns: (transfer full): a #GListModel of #IdeSearchResult
  */
-GPtrArray *
+GListModel *
 ide_search_provider_search_finish (IdeSearchProvider  *self,
                                    GAsyncResult       *result,
+                                   gboolean           *truncated,
                                    GError            **error)
 {
+  GListModel *ret;
+  gboolean empty_trunicated;
+
+  IDE_ENTRY;
+
+  g_return_val_if_fail (IDE_IS_MAIN_THREAD (), NULL);
   g_return_val_if_fail (IDE_IS_SEARCH_PROVIDER (self), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
 
-  return IDE_SEARCH_PROVIDER_GET_IFACE (self)->search_finish (self, result, error);
+  if (truncated == NULL)
+    truncated = &empty_trunicated;
+
+  *truncated = FALSE;
+
+  ret = IDE_SEARCH_PROVIDER_GET_IFACE (self)->search_finish (self, result, truncated, error);
+
+  g_return_val_if_fail (!ret || G_IS_LIST_MODEL (ret), NULL);
+  g_return_val_if_fail (*truncated == TRUE || *truncated == FALSE, NULL);
+
+  IDE_RETURN (ret);
 }
