@@ -1,6 +1,6 @@
 /* ide-source-style-scheme.h
  *
- * Copyright 2016 Christian Hergert <chergert@redhat.com>
+ * Copyright 2022 Christian Hergert <chergert@redhat.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,19 +14,27 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #pragma once
 
+#if !defined (IDE_SOURCEVIEW_INSIDE) && !defined (IDE_SOURCEVIEW_COMPILATION)
+# error "Only <libide-sourceview.h> can be included directly."
+#endif
+
 #include <gtksourceview/gtksource.h>
 
-#include "ide-version-macros.h"
+#include <libide-core.h>
 
 G_BEGIN_DECLS
 
 IDE_AVAILABLE_IN_ALL
-gboolean ide_source_style_scheme_apply_style (GtkSourceStyleScheme *style_scheme,
-                                              const gchar          *style,
-                                              GtkTextTag           *tag);
+gboolean              ide_source_style_scheme_is_dark     (GtkSourceStyleScheme *scheme);
+IDE_AVAILABLE_IN_ALL
+GtkSourceStyleScheme *ide_source_style_scheme_get_variant (GtkSourceStyleScheme *scheme,
+                                                           const char           *variant);
+
 
 G_END_DECLS
