@@ -24,16 +24,19 @@
 #include "config.h"
 
 #include <fcntl.h>
-#include <glib/gi18n.h>
 #include <pcre2.h>
 #include <stdlib.h>
-#include <vte/vte.h>
 #include <unistd.h>
 
+#include <glib/gi18n.h>
+#include <vte/vte.h>
+
+#include <libide-gtk.h>
+
 #include "ide-terminal-search.h"
 #include "ide-terminal-search-private.h"
 
-G_DEFINE_TYPE (IdeTerminalSearch, ide_terminal_search, GTK_TYPE_BIN)
+G_DEFINE_FINAL_TYPE (IdeTerminalSearch, ide_terminal_search, ADW_TYPE_BIN)
 
 enum {
   PROP_0,
@@ -101,15 +104,15 @@ update_regex (IdeTerminalSearch *self)
 
   g_assert (IDE_IS_TERMINAL_SEARCH (self));
 
-  search_text = gtk_entry_get_text (GTK_ENTRY (self->search_entry));
-  caseless = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->match_case_checkbutton));
+  search_text = gtk_editable_get_text (GTK_EDITABLE (self->search_entry));
+  caseless = !gtk_check_button_get_active (GTK_CHECK_BUTTON (self->match_case_checkbutton));
 
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->regex_checkbutton)))
+  if (gtk_check_button_get_active (GTK_CHECK_BUTTON (self->regex_checkbutton)))
     pattern = g_strdup (search_text);
   else
     pattern = g_regex_escape_string (search_text, -1);
 
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->entire_word_checkbutton)))
+  if (gtk_check_button_get_active (GTK_CHECK_BUTTON (self->entire_word_checkbutton)))
     {
       char *new_pattern;
       new_pattern = g_strdup_printf ("\\b%s\\b", pattern);
@@ -117,15 +120,10 @@ update_regex (IdeTerminalSearch *self)
       pattern = new_pattern;
     }
 
-  if (self->regex_caseless == caseless &&
-      g_strcmp0 (self->regex_pattern, pattern) == 0)
+  if (self->regex_caseless == caseless && g_strcmp0 (self->regex_pattern, pattern) == 0)
     return;
 
-  if (self->regex)
-    {
-      vte_regex_unref (self->regex);
-    }
-
+  g_clear_pointer (&self->regex, vte_regex_unref);
   g_clear_pointer (&self->regex_pattern, g_free);
 
   if (search_text[0] != '\0')
@@ -141,8 +139,6 @@ update_regex (IdeTerminalSearch *self)
       if (self->regex != NULL)
         self->regex_pattern = g_steal_pointer (&pattern);
     }
-  else
-    self->regex = NULL;
 
   update_sensitivity (self);
 
@@ -150,8 +146,9 @@ update_regex (IdeTerminalSearch *self)
 }
 
 static void
-search_text_changed_cb (IdeTaggedEntry  *search_entry,
-                        IdeTerminalSearch *self)
+search_notify_text_cb (IdeSearchEntry    *search_entry,
+                       GParamSpec        *pspec,
+                       IdeTerminalSearch *self)
 {
   update_regex (self);
 }
@@ -170,18 +167,6 @@ wrap_around_toggled_cb (GtkToggleButton *button,
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_WRAP_AROUND]);
 }
 
-static void
-reveal_options_changed_cb (GtkToggleButton *button,
-                           IdeTerminalSearch  *self)
-{
-  g_assert (IDE_IS_TERMINAL_SEARCH (self));
-
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->reveal_button)))
-    gtk_widget_set_visible (GTK_WIDGET (self->search_options), TRUE);
-  else
-    gtk_widget_set_visible (GTK_WIDGET (self->search_options), FALSE);
-}
-
 static void
 search_overlay_notify_regex_cb (VteTerminal    *terminal,
                                 GParamSpec     *pspec G_GNUC_UNUSED,
@@ -211,8 +196,8 @@ search_overlay_notify_wrap_around_cb (VteTerminal    *terminal,
 }
 
 static void
-search_overlay_search_cb (VteTerminal    *terminal,
-                          gboolean        backward,
+search_overlay_search_cb (VteTerminal       *terminal,
+                          gboolean           backward,
                           IdeTerminalSearch *self)
 {
   g_assert (VTE_IS_TERMINAL (terminal));
@@ -232,12 +217,19 @@ search_revealer_cb (GtkRevealer       *search_revealer,
 
   if (gtk_revealer_get_child_revealed (search_revealer))
     {
+#if 0
       if (vte_terminal_get_has_selection (self->terminal))
         {
           vte_terminal_copy_primary (self->terminal);
-          self->selected_text = gtk_clipboard_wait_for_text (self->clipboard);
-          gtk_entry_set_text (GTK_ENTRY (self->search_entry), self->selected_text);
+
+          g_clear_pointer (&self->selected_text, g_free);
+
+          /* TODO: Wait for async text read */
+
+          gtk_editable_set_text (GTK_EDITABLE (self->search_entry), self->selected_text);
         }
+#endif
+
       gtk_widget_grab_focus (GTK_WIDGET (self->search_entry));
     }
   else
@@ -246,6 +238,17 @@ search_revealer_cb (GtkRevealer       *search_revealer,
     }
 }
 
+static void
+search_hide_action (GtkWidget  *widget,
+                    const char *action_name,
+                    GVariant   *param)
+{
+  IdeTerminalSearch *self = IDE_TERMINAL_SEARCH (widget);
+
+  gtk_revealer_set_reveal_child (self->search_revealer, FALSE);
+  gtk_widget_grab_focus (GTK_WIDGET (self->terminal));
+}
+
 static void
 ide_terminal_search_connect_terminal (IdeTerminalSearch *self)
 {
@@ -312,6 +315,8 @@ ide_terminal_search_class_init (IdeTerminalSearchClass *klass)
   gtk_widget_class_bind_template_child (widget_class, IdeTerminalSearch, search_revealer);
   gtk_widget_class_bind_template_child (widget_class, IdeTerminalSearch, search_options);
 
+  gtk_widget_class_install_action (widget_class, "search.hide", NULL, search_hide_action);
+
   signals[SEARCH] =
     g_signal_new ("search",
                   G_OBJECT_CLASS_TYPE (object_class),
@@ -341,18 +346,15 @@ ide_terminal_search_init (IdeTerminalSearch *self)
   self->regex_caseless = FALSE;
   self->regex_pattern = 0;
 
-  self->clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
-
   gtk_widget_init_template (GTK_WIDGET (self));
 
   g_signal_connect (self->search_prev_button, "clicked", G_CALLBACK (search_button_clicked_cb), self);
   g_signal_connect (self->search_next_button, "clicked", G_CALLBACK (search_button_clicked_cb), self);
   g_signal_connect (self->close_button, "clicked", G_CALLBACK (close_clicked_cb), self);
-  g_signal_connect (self->search_entry, "search-changed", G_CALLBACK (search_text_changed_cb), self);
+  g_signal_connect (self->search_entry, "notify::text", G_CALLBACK (search_notify_text_cb), self);
   g_signal_connect (self->match_case_checkbutton, "toggled", G_CALLBACK (search_parameters_changed_cb), self);
   g_signal_connect (self->entire_word_checkbutton, "toggled", G_CALLBACK (search_parameters_changed_cb), self);
   g_signal_connect (self->regex_checkbutton, "toggled", G_CALLBACK (search_parameters_changed_cb), self);
-  g_signal_connect (self->reveal_button, "toggled", G_CALLBACK (reveal_options_changed_cb), self);
   g_signal_connect (self->wrap_around_checkbutton, "toggled", G_CALLBACK (wrap_around_toggled_cb), self);
   g_signal_connect (self->search_revealer, "notify::child-revealed", G_CALLBACK (search_revealer_cb), self);
 }
@@ -360,8 +362,6 @@ ide_terminal_search_init (IdeTerminalSearch *self)
 /**
  * ide_terminal_search_set_terminal:
  * @self: a #IdeTerminalSearch
- *
- * Since: 3.32
  */
 void
 ide_terminal_search_set_terminal (IdeTerminalSearch *self,
@@ -378,8 +378,6 @@ ide_terminal_search_set_terminal (IdeTerminalSearch *self,
  * @self: a #IdeTerminalSearch
  *
  * Returns: (transfer none) (nullable): a #VteRegex or %NULL.
- *
- * Since: 3.32
  */
 VteRegex *
 ide_terminal_search_get_regex (IdeTerminalSearch *self)
@@ -394,14 +392,13 @@ ide_terminal_search_get_regex (IdeTerminalSearch *self)
  * @self: a #IdeTerminalSearch
  *
  *
- * Since: 3.32
  */
 gboolean
 ide_terminal_search_get_wrap_around (IdeTerminalSearch *self)
 {
   g_return_val_if_fail (IDE_IS_TERMINAL_SEARCH (self), FALSE);
 
-  return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->wrap_around_checkbutton));
+  return gtk_check_button_get_active (GTK_CHECK_BUTTON (self->wrap_around_checkbutton));
 }
 
 /**
@@ -411,8 +408,6 @@ ide_terminal_search_get_wrap_around (IdeTerminalSearch *self)
  * Gets the revealer widget used for the terminal search.
  *
  * Returns: (transfer none): a #GtkRevealer
- *
- * Since: 3.32
  */
 GtkRevealer *
 ide_terminal_search_get_revealer (IdeTerminalSearch *self)
