@@ -1,6 +1,6 @@
 /* main.c
  *
- * Copyright 2018-2019 Christian Hergert <chergert@redhat.com>
+ * Copyright 2018-2022 Christian Hergert <chergert@redhat.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,26 +22,35 @@
 
 #include "config.h"
 
-#include <girepository.h>
-#include <glib/gi18n.h>
-#include <gtksourceview/gtksource.h>
-#include <libide-core.h>
-#include <libide-code.h>
-#include <libide-editor.h>
-#include <libide-greeter.h>
-#include <libide-gui.h>
-#include <libide-threading.h>
 #include <locale.h>
+#include <sched.h>
+#include <stdlib.h>
+#include <unistd.h>
+
+#include <glib/gi18n.h>
+
+#include <girepository.h>
+#include <gtksourceview/gtksource.h>
+
 #ifdef ENABLE_TRACING_SYSCAP
 # include <sysprof-capture.h>
 #endif
-#include <sched.h>
-#include <unistd.h>
+
+#include <libide-core.h>
+#include <libide-code.h>
+#include <libide-gui.h>
+#include <libide-threading.h>
 
 #include "ide-application-private.h"
 #include "ide-build-ident.h"
-#include "ide-thread-private.h"
+#include "ide-editor-private.h"
+#include "ide-gtk-private.h"
+#include "ide-log-private.h"
+#include "ide-search-private.h"
+#include "ide-shell-private.h"
 #include "ide-terminal-private.h"
+#include "ide-thread-private.h"
+#include "ide-tweaks-init.h"
 #include "ide-private.h"
 
 #include "bug-buddy.h"
@@ -137,19 +146,13 @@ verbose_cb (const gchar  *option_name,
 static void
 early_params_check (gint       *argc,
                     gchar    ***argv,
-                    gboolean   *standalone,
-                    gchar     **type,
-                    gchar     **plugin,
-                    gchar     **dbus_address)
+                    gboolean   *standalone)
 {
   g_autoptr(GOptionContext) context = NULL;
   g_autoptr(GOptionGroup) gir_group = NULL;
   GOptionEntry entries[] = {
     { "standalone", 's', 0, G_OPTION_ARG_NONE, standalone, N_("Run a new instance of Builder") },
     { "verbose", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, verbose_cb },
-    { "plugin", 0, 0, G_OPTION_ARG_STRING, plugin },
-    { "type", 0, 0, G_OPTION_ARG_STRING, type },
-    { "dbus-address", 0, 0, G_OPTION_ARG_STRING, dbus_address },
     { NULL }
   };
 
@@ -209,14 +212,18 @@ gint
 main (gint   argc,
       gchar *argv[])
 {
-  g_autofree gchar *plugin = NULL;
-  g_autofree gchar *type = NULL;
-  g_autofree gchar *dbus_address = NULL;
+  g_autofree char *messages_debug = NULL;
   IdeApplication *app;
   const gchar *desktop;
   gboolean standalone = FALSE;
   int ret;
 
+  /* Get environment variable early and clear it from GLib. We want to be
+   * certain we don't pass this on to child processes so we clear it upfront.
+   */
+  messages_debug = g_strdup (getenv ("G_MESSAGES_DEBUG"));
+  unsetenv ("G_MESSAGES_DEBUG");
+
   /* Setup our gdb fork()/exec() helper if we're in a terminal */
   if (is_running_in_shell ())
     bug_buddy_init ();
@@ -234,21 +241,18 @@ main (gint   argc,
   g_set_prgname (ide_get_program_name ());
   g_set_application_name (_("Builder"));
 
-#if 0
-  /* TODO: allow support for parallel nightly install */
 #ifdef DEVELOPMENT_BUILD
-  ide_set_application_id ("org.gnome.Builder-Devel");
-#endif
+  ide_set_application_id ("org.gnome.Builder.Devel");
 #endif
 
   /* Early init of logging so that we get messages in a consistent
    * format. If we deferred this to GApplication, we'd get them in
    * multiple formats.
    */
-  ide_log_init (TRUE, NULL);
+  ide_log_init (TRUE, NULL, messages_debug);
 
   /* Extract options like -vvvv */
-  early_params_check (&argc, &argv, &standalone, &type, &plugin, &dbus_address);
+  early_params_check (&argc, &argv, &standalone);
 
   /* Log some info so it shows up in logs */
   g_message ("GNOME Builder %s (%s) from channel \"%s\" starting with ABI %s",
@@ -284,15 +288,26 @@ main (gint   argc,
              gtk_get_minor_version (),
              gtk_get_micro_version ());
 
+  /* Init libraries with initializers */
+  gtk_init ();
   gtk_source_init ();
+  adw_init ();
+  panel_init ();
 
   /* Initialize thread pools */
   _ide_thread_pool_init (FALSE);
 
-  /* Guess the user shell early */
-  _ide_guess_shell ();
+  /* Guess the user $SHELL and $PATH early */
+  _ide_shell_init ();
 
-  app = _ide_application_new (standalone, type, plugin, dbus_address);
+  /* Ensure availability of some symbols possibly dropped in link */
+  _ide_tweaks_init ();
+  _ide_gtk_init ();
+  _ide_search_init ();
+  _ide_editor_init ();
+  _ide_terminal_init ();
+
+  app = _ide_application_new (standalone);
   g_application_add_option_group (G_APPLICATION (app), g_irepository_get_option_group ());
   ret = g_application_run (G_APPLICATION (app), argc, argv);
   /* Force disposal of the application (to help catch cleanup
