@@ -25,26 +25,21 @@
 #include <libpeas/peas.h>
 #include <libide-editor.h>
 #include <libide-gui.h>
-#include <libide-tree.h>
 
-#include "gbp-buildui-config-view-addin.h"
 #include "gbp-buildui-editor-page-addin.h"
+#include "gbp-buildui-tweaks-addin.h"
 #include "gbp-buildui-workspace-addin.h"
-#include "gbp-buildui-tree-addin.h"
 
 _IDE_EXTERN void
 _gbp_buildui_register_types (PeasObjectModule *module)
 {
-  peas_object_module_register_extension_type (module,
-                                              IDE_TYPE_CONFIG_VIEW_ADDIN,
-                                              GBP_TYPE_BUILDUI_CONFIG_VIEW_ADDIN);
   peas_object_module_register_extension_type (module,
                                               IDE_TYPE_EDITOR_PAGE_ADDIN,
                                               GBP_TYPE_BUILDUI_EDITOR_PAGE_ADDIN);
+  peas_object_module_register_extension_type (module,
+                                              IDE_TYPE_TWEAKS_ADDIN,
+                                              GBP_TYPE_BUILDUI_TWEAKS_ADDIN);
   peas_object_module_register_extension_type (module,
                                               IDE_TYPE_WORKSPACE_ADDIN,
                                               GBP_TYPE_BUILDUI_WORKSPACE_ADDIN);
-  peas_object_module_register_extension_type (module,
-                                              IDE_TYPE_TREE_ADDIN,
-                                              GBP_TYPE_BUILDUI_TREE_ADDIN);
 }
