@@ -35,20 +35,31 @@ populate_cb (GObject      *object,
   IdeCodeIndexIndex *index = (IdeCodeIndexIndex *)object;
   g_autoptr(IdeTask) task = user_data;
   g_autoptr(GPtrArray) results = NULL;
+  g_autoptr(GListStore) store = NULL;
   g_autoptr(GError) error = NULL;
+  gboolean truncated = FALSE;
 
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (IDE_IS_CODE_INDEX_INDEX (index));
   g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (IDE_IS_TASK (task));
 
-  results = ide_code_index_index_populate_finish (index, result, &error);
+  if (!(results = ide_code_index_index_populate_finish (index, result, &truncated, &error)))
+    {
+      ide_task_return_error (task, g_steal_pointer (&error));
+      IDE_EXIT;
+    }
 
-  if (results != NULL)
-    ide_task_return_pointer (task,
-                             g_steal_pointer (&results),
-                             (GDestroyNotify)g_ptr_array_unref);
-  else
-    ide_task_return_error (task, g_steal_pointer (&error));
+  if (truncated)
+    g_object_set_data (G_OBJECT (task), "TRUNCATED", GINT_TO_POINTER (truncated));
+
+  store = g_list_store_new (IDE_TYPE_SEARCH_RESULT);
+  g_list_store_splice (store, 0, 0, results->pdata, results->len);
+  ide_task_return_pointer (task, g_steal_pointer (&store), g_object_unref);
+
+  IDE_EXIT;
 }
 
 static void
@@ -106,12 +117,13 @@ ide_code_index_search_provider_search_async (IdeSearchProvider   *provider,
   IDE_EXIT;
 }
 
-static GPtrArray *
-ide_code_index_search_provider_search_finish (IdeSearchProvider *provider,
-                                              GAsyncResult      *result,
-                                              GError           **error)
+static GListModel *
+ide_code_index_search_provider_search_finish (IdeSearchProvider  *provider,
+                                              GAsyncResult       *result,
+                                              gboolean           *truncated,
+                                              GError            **error)
 {
-  GPtrArray *ar;
+  GListModel *ret;
 
   IDE_ENTRY;
 
@@ -119,9 +131,13 @@ ide_code_index_search_provider_search_finish (IdeSearchProvider *provider,
   g_return_val_if_fail (IDE_IS_CODE_INDEX_SEARCH_PROVIDER (provider), NULL);
   g_return_val_if_fail (IDE_IS_TASK (result), NULL);
 
-  ar = ide_task_propagate_pointer (IDE_TASK (result), error);
+  *truncated = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (result), "TRUNCATED"));
 
-  IDE_RETURN (IDE_PTR_ARRAY_STEAL_FULL (&ar));
+  ret = ide_task_propagate_pointer (IDE_TASK (result), error);
+
+  g_return_val_if_fail (!ret || G_IS_LIST_MODEL (ret), NULL);
+
+  IDE_RETURN (ret);
 }
 
 static void
@@ -133,7 +149,7 @@ search_provider_iface_init (IdeSearchProviderInterface *iface)
 
 struct _IdeCodeIndexSearchProvider { IdeObject parent; };
 
-G_DEFINE_TYPE_WITH_CODE (IdeCodeIndexSearchProvider,
+G_DEFINE_FINAL_TYPE_WITH_CODE (IdeCodeIndexSearchProvider,
                          ide_code_index_search_provider,
                          IDE_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (IDE_TYPE_SEARCH_PROVIDER, search_provider_iface_init))
