@@ -20,12 +20,14 @@
 
 #pragma once
 
+#include <libide-foundry.h>
 #include <libide-threading.h>
 
 G_BEGIN_DECLS
 
-void gbp_flatpak_aux_init  (void);
-int  gbp_flatpak_aux_apply (IdeSubprocessLauncher *launcher,
-                            int                    position);
+void gbp_flatpak_aux_init                  (void);
+int  gbp_flatpak_aux_apply                 (IdeSubprocessLauncher *launcher,
+                                            int                    position);
+void gbp_flatpak_aux_append_to_run_context (IdeRunContext         *run_context);
 
 G_END_DECLS
