@@ -256,14 +256,14 @@ reload_manifest_cb (GObject      *object,
   is_active = current == IDE_CONFIG (old_manifest);
 
   ide_config_provider_emit_added (IDE_CONFIG_PROVIDER (self),
-                                         IDE_CONFIG (new_manifest));
+                                  IDE_CONFIG (new_manifest));
 
   if (is_active)
     ide_config_manager_set_current (manager,
-                                           IDE_CONFIG (new_manifest));
+                                    IDE_CONFIG (new_manifest));
 
   ide_config_provider_emit_removed (IDE_CONFIG_PROVIDER (self),
-                                           IDE_CONFIG (old_manifest));
+                                    IDE_CONFIG (old_manifest));
 }
 
 static void
@@ -481,7 +481,7 @@ gbp_flatpak_config_provider_load_async (IdeConfigProvider   *provider,
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
   context = ide_object_get_context (IDE_OBJECT (self));
-  client = gbp_flatpak_client_from_context (context);
+  client = gbp_flatpak_client_get_default ();
   service = gbp_flatpak_client_get_service (client, NULL, NULL);
   vcs = ide_vcs_from_context (context);
   workdir = ide_vcs_get_workdir (vcs);
@@ -535,7 +535,7 @@ guess_best_config (GPtrArray *ar)
       /* If appid.json is the same as the filename, that is the
        * best match (after unstable) we can have. Use it.
        */
-      if (dzl_str_equal0 (app_id_json, base))
+      if (ide_str_equal0 (app_id_json, base))
         return IDE_CONFIG (config);
     }
 
