@@ -21,11 +21,16 @@
 
 #define G_LOG_DOMAIN "gbp-flatpak-manifest"
 
+#include "config.h"
+
+#include <glib/gi18n.h>
+
 #include <json-glib/json-glib.h>
 
 #include "gbp-flatpak-client.h"
 #include "gbp-flatpak-manifest.h"
 #include "gbp-flatpak-runtime.h"
+#include "gbp-flatpak-sdk.h"
 #include "gbp-flatpak-util.h"
 
 #include "daemon/ipc-flatpak-service.h"
@@ -33,7 +38,7 @@
 
 struct _GbpFlatpakManifest
 {
-  IdeConfig  parent_instance;
+  IdeConfig         parent_instance;
 
   GFile            *file;
   GFileMonitor     *file_monitor;
@@ -45,6 +50,7 @@ struct _GbpFlatpakManifest
    */
   gchar           **build_args;
   gchar            *command;
+  gchar           **x_run_args;
   gchar           **finish_args;
   gchar            *runtime;
   gchar            *runtime_version;
@@ -65,7 +71,7 @@ struct _GbpFlatpakManifest
 static void initable_iface_init (GInitableIface *iface);
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (GbpFlatpakManifest, gbp_flatpak_manifest, IDE_TYPE_CONFIG,
-                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, initable_iface_init))
+                               G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, initable_iface_init))
 
 enum {
   PROP_0,
@@ -81,6 +87,125 @@ enum {
 static GParamSpec *properties [N_PROPS];
 static guint signals [N_SIGNALS];
 
+static GbpFlatpakSdk *
+get_sdk (IdeSdkManager *sdk_manager,
+         const char    *id)
+{
+  g_autofree char *full_id = NULL;
+  guint n_items;
+
+  g_assert (IDE_IS_SDK_MANAGER (sdk_manager));
+  g_assert (id != NULL);
+
+  full_id = g_strdup_printf ("runtime/%s", id);
+  n_items = g_list_model_get_n_items (G_LIST_MODEL (sdk_manager));
+
+  for (guint i = 0; i < n_items; i++)
+    {
+      g_autoptr(IdeSdk) sdk = g_list_model_get_item (G_LIST_MODEL (sdk_manager), i);
+      const char *sdk_id;
+
+      if (!GBP_IS_FLATPAK_SDK (sdk))
+        continue;
+
+      if ((sdk_id = gbp_flatpak_sdk_get_id (GBP_FLATPAK_SDK (sdk))) &&
+          ide_str_equal0 (full_id, sdk_id))
+        return GBP_FLATPAK_SDK (g_steal_pointer (&sdk));
+    }
+
+  return NULL;
+}
+
+static GFile *
+gbp_flatpak_manifest_translate_file (IdeConfig *config,
+                                     GFile     *file)
+{
+  GbpFlatpakManifest *self = (GbpFlatpakManifest *)config;
+  g_autoptr(GbpFlatpakSdk) platform = NULL;
+  g_autofree char *id = NULL;
+  g_autofree char *arch = NULL;
+  IdeBuildManager *build_manager;
+  IdeSdkManager *sdk_manager;
+  IdePipeline *pipeline;
+  IdeContext *context;
+  GFile *translated;
+  const char *path;
+
+  g_assert (GBP_IS_FLATPAK_MANIFEST (self));
+  g_assert (G_IS_FILE (file));
+
+  /* FIXME:
+   *
+   * This could probably all be improved by creating a "translation object"
+   * which can be backend specific so that it can cache this state without
+   * so much overhead.
+   */
+
+  /* We can only translate native paths */
+  if (!g_file_is_native (file) || !(path = g_file_peek_path (file)))
+    return g_object_ref (file);
+
+  context = ide_object_get_context (IDE_OBJECT (config));
+  build_manager = ide_build_manager_from_context (context);
+  pipeline = ide_build_manager_get_pipeline (build_manager);
+  sdk_manager = ide_sdk_manager_get_default ();
+
+  /* If this is /app/, then it's in the staging directory */
+  if (g_str_equal (path, "/app") || g_str_has_prefix (path, "/app/"))
+    {
+      g_autofree char *staging_dir = gbp_flatpak_get_staging_dir (pipeline);
+
+      if (path[4] == 0 || !(path = g_path_skip_root (path + 4)) || path[0] == 0)
+        path = NULL;
+
+      return g_file_new_build_filename (staging_dir, "files", path, NULL);
+    }
+
+  if (pipeline != NULL)
+    arch = ide_pipeline_get_arch (pipeline);
+  else
+    arch = g_strdup (ide_get_system_arch ());
+
+  /* Start with SDK-extensions, since they tend to have the
+   * Longest Common Prefix with the target.
+   */
+  if (self->sdk_extensions != NULL)
+    {
+      for (guint i = 0; self->sdk_extensions[i]; i++)
+        {
+          g_autoptr(GbpFlatpakSdk) sdk_extension = get_sdk (sdk_manager, self->sdk_extensions[i]);
+
+          if (sdk_extension != NULL &&
+              (translated = gbp_flatpak_sdk_translate_file (sdk_extension, file)))
+            return g_steal_pointer (&translated);
+        }
+    }
+
+  if (self->sdk != NULL)
+    {
+      g_autofree char *sdk_id = g_strdup_printf ("%s/%s/%s", self->sdk, arch, self->runtime_version);
+      g_autofree char *sdk_debug_id = g_strdup_printf ("%s.Debug/%s/%s", self->sdk, arch, self->runtime_version);
+      g_autoptr(GbpFlatpakSdk) sdk = NULL;
+      g_autoptr(GbpFlatpakSdk) sdk_debug = NULL;
+
+      if (g_str_has_prefix (path, "/usr/lib/debug/") &&
+          ((sdk_debug = get_sdk (sdk_manager, sdk_debug_id)) &&
+           (translated = gbp_flatpak_sdk_translate_file (sdk_debug, file))))
+        return g_steal_pointer (&translated);
+
+      if ((sdk = get_sdk (sdk_manager, sdk_id)) &&
+          (translated = gbp_flatpak_sdk_translate_file (sdk, file)))
+        return g_steal_pointer (&translated);
+    }
+
+  id = g_strdup_printf ("%s/%s/%s", self->runtime, arch, self->runtime_version);
+  if ((platform = get_sdk (sdk_manager, id)) &&
+      (translated = gbp_flatpak_sdk_translate_file (platform, file)))
+    return g_steal_pointer (&translated);
+
+  return NULL;
+}
+
 static gboolean
 validate_properties (GbpFlatpakManifest  *self,
                      GError             **error)
@@ -168,6 +293,7 @@ discover_strv_field (JsonObject    *object,
 
       g_ptr_array_add (ar, NULL);
 
+      g_clear_pointer (location, g_strfreev);
       *location = (gchar **)g_ptr_array_free (g_steal_pointer (&ar), FALSE);
 
       return TRUE;
@@ -209,6 +335,26 @@ discover_strv_as_quoted (JsonObject   *object,
   return FALSE;
 }
 
+static void
+discover_build_options (GbpFlatpakManifest *self,
+                        JsonObject         *root)
+{
+  JsonObject *build_options;
+
+  g_assert (GBP_IS_FLATPAK_MANIFEST (self));
+  g_assert (root != NULL);
+
+  /* TODO: this all needs a revamp, as these can be changed per arch */
+
+  if (!json_object_has_member (root, "build-options"))
+    return;
+
+  if (!(build_options = json_object_get_object_member (root, "build-options")))
+    return;
+
+  discover_strv_field (build_options, "build-args", &self->build_args);
+}
+
 static void
 discover_environ (GbpFlatpakManifest *self,
                   JsonObject         *root)
@@ -296,7 +442,7 @@ discover_primary_module (GbpFlatpakManifest  *self,
           !(name = json_object_get_string_member (obj, "name")))
         continue;
 
-      if (dzl_str_equal0 (name, dir_name))
+      if (ide_str_equal0 (name, dir_name))
         {
           self->primary_module = g_strdup (name);
           return obj;
@@ -422,9 +568,9 @@ gbp_flatpak_manifest_initable_init (GInitable     *initable,
   discover_string_field (root_obj, "base-version", &self->base_version);
   discover_string_field (root_obj, "sdk", &self->sdk);
   discover_string_field (root_obj, "command", &self->command);
-  discover_strv_field (root_obj, "build-args", &self->build_args);
   discover_strv_field (root_obj, "finish-args", &self->finish_args);
   discover_strv_field (root_obj, "sdk-extensions", &self->sdk_extensions);
+  discover_strv_field (root_obj, "x-run-args", &self->x_run_args);
 
   if (discover_strv_as_quoted (root_obj, "x-run-args", &run_args))
     ide_config_set_run_opts (IDE_CONFIG (self), run_args);
@@ -477,6 +623,7 @@ gbp_flatpak_manifest_initable_init (GInitable     *initable,
                                    (const gchar * const *)make_install_args);
 
   discover_environ (self, root_obj);
+  discover_build_options (self, root_obj);
 
   self->root = json_node_ref (root);
   self->primary = json_object_ref (primary);
@@ -607,7 +754,8 @@ find_extension (GbpFlatpakManifest *self,
                                  &deploy_dir,
                                  &metadata,
                                  &is_extension))
-        ret = gbp_flatpak_runtime_new (name,
+        ret = gbp_flatpak_runtime_new (IDE_OBJECT (self),
+                                       name,
                                        arch,
                                        branch,
                                        sdk_name,
@@ -650,6 +798,12 @@ gbp_flatpak_manifest_get_extensions (IdeConfig *config)
   return g_steal_pointer (&ret);
 }
 
+static char *
+gbp_flatpak_manifest_get_description (IdeConfig *config)
+{
+  return g_strdup (_("Flatpak"));
+}
+
 static void
 gbp_flatpak_manifest_finalize (GObject *object)
 {
@@ -727,6 +881,8 @@ gbp_flatpak_manifest_class_init (GbpFlatpakManifestClass *klass)
 
   config_class->get_extensions = gbp_flatpak_manifest_get_extensions;
   config_class->supports_runtime = gbp_flatpak_manifest_supports_runtime;
+  config_class->get_description = gbp_flatpak_manifest_get_description;
+  config_class->translate_file = gbp_flatpak_manifest_translate_file;
 
   properties [PROP_FILE] =
     g_param_spec_object ("file",
@@ -932,9 +1088,9 @@ apply_changes_to_tree (GbpFlatpakManifest *self)
       key = ide_environment_variable_get_key (var);
       value = ide_environment_variable_get_value (var);
 
-      if (dzl_str_equal0 (key, "CFLAGS"))
+      if (ide_str_equal0 (key, "CFLAGS"))
         json_object_set_string_member (build_options, "cflags", value);
-      else if (dzl_str_equal0 (key, "CXXFLAGS"))
+      else if (ide_str_equal0 (key, "CXXFLAGS"))
         json_object_set_string_member (build_options, "cxxflags", value);
       else
         json_object_set_string_member (env_obj, key, value);
@@ -1213,3 +1369,11 @@ gbp_flatpak_manifest_get_branch (GbpFlatpakManifest *self)
 
   return self->runtime_version;
 }
+
+const char * const *
+gbp_flatpak_manifest_get_x_run_args (GbpFlatpakManifest *self)
+{
+  g_return_val_if_fail (GBP_IS_FLATPAK_MANIFEST (self), NULL);
+
+  return (const char * const *)self->x_run_args;
+}
