@@ -22,11 +22,12 @@
 
 #include <glib/gi18n.h>
 
+#include <libide-gtk.h>
+
 #include "gbp-flatpak-manifest.h"
 #include "gbp-flatpak-download-stage.h"
 #include "gbp-flatpak-pipeline-addin.h"
 #include "gbp-flatpak-runtime.h"
-#include "gbp-flatpak-transfer.h"
 #include "gbp-flatpak-util.h"
 
 #define VERSION_CHECK(v,a,b,c)                                         \
@@ -148,11 +149,15 @@ static IdeSubprocessLauncher *
 create_subprocess_launcher (void)
 {
   IdeSubprocessLauncher *launcher;
+  const char *config_dir;
+
+  config_dir = gbp_flatpak_get_config_dir ();
 
   launcher = ide_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDOUT_PIPE |
                                           G_SUBPROCESS_FLAGS_STDERR_PIPE);
   ide_subprocess_launcher_set_run_on_host (launcher, TRUE);
   ide_subprocess_launcher_set_clear_env (launcher, FALSE);
+  ide_subprocess_launcher_setenv (launcher, "FLATPAK_CONFIG_DIR", config_dir, TRUE);
 
   return launcher;
 }
@@ -193,17 +198,17 @@ reap_staging_dir_cb (GObject      *object,
                      GAsyncResult *result,
                      gpointer      user_data)
 {
-  DzlDirectoryReaper *reaper = (DzlDirectoryReaper *)object;
+  IdeDirectoryReaper *reaper = (IdeDirectoryReaper *)object;
   g_autoptr(IdePipelineStage) stage = user_data;
   g_autoptr(GError) error = NULL;
 
   IDE_ENTRY;
 
-  g_assert (DZL_IS_DIRECTORY_REAPER (reaper));
+  g_assert (IDE_IS_DIRECTORY_REAPER (reaper));
   g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (IDE_IS_PIPELINE_STAGE (stage));
 
-  if (!dzl_directory_reaper_execute_finish (reaper, result, &error))
+  if (!ide_directory_reaper_execute_finish (reaper, result, &error))
     ide_object_warning (stage,
                         "Failed to reap staging directory: %s",
                         error->message);
@@ -255,14 +260,14 @@ check_for_build_init_files (IdePipelineStage    *stage,
 
   if (!completed && parent_exists)
     {
-      g_autoptr(DzlDirectoryReaper) reaper = NULL;
+      g_autoptr(IdeDirectoryReaper) reaper = NULL;
       g_autoptr(GFile) staging = g_file_new_for_path (staging_dir);
 
       ide_pipeline_stage_pause (stage);
 
-      reaper = dzl_directory_reaper_new ();
-      dzl_directory_reaper_add_directory (reaper, staging, 0);
-      dzl_directory_reaper_execute_async (reaper,
+      reaper = ide_directory_reaper_new ();
+      ide_directory_reaper_add_directory (reaper, staging, 0);
+      ide_directory_reaper_execute_async (reaper,
                                           cancellable,
                                           reap_staging_dir_cb,
                                           g_object_ref (stage));
@@ -271,22 +276,22 @@ check_for_build_init_files (IdePipelineStage    *stage,
 
 static void
 reap_staging_dir (IdePipelineStage      *stage,
-                  DzlDirectoryReaper *reaper,
+                  IdeDirectoryReaper *reaper,
                   const gchar        *staging_dir)
 {
   g_autoptr(GFile) dir = NULL;
 
   g_assert (IDE_IS_PIPELINE_STAGE (stage));
-  g_assert (DZL_IS_DIRECTORY_REAPER (reaper));
+  g_assert (IDE_IS_DIRECTORY_REAPER (reaper));
   g_assert (staging_dir != NULL);
 
   dir = g_file_new_for_path (staging_dir);
-  dzl_directory_reaper_add_directory (reaper, dir, 0);
+  ide_directory_reaper_add_directory (reaper, dir, 0);
 }
 
 static gboolean
 register_build_init_stage (GbpFlatpakPipelineAddin  *self,
-                           IdePipeline         *pipeline,
+                           IdePipeline              *pipeline,
                            IdeContext               *context,
                            GError                  **error)
 {
@@ -295,6 +300,7 @@ register_build_init_stage (GbpFlatpakPipelineAddin  *self,
   g_autofree gchar *staging_dir = NULL;
   g_autofree gchar *sdk = NULL;
   g_autofree gchar *arch = NULL;
+  const char * const *sdk_extensions = NULL;
   IdeConfig *config;
   IdeRuntime *runtime;
   const gchar *app_id;
@@ -327,11 +333,14 @@ register_build_init_stage (GbpFlatpakPipelineAddin  *self,
   sdk = gbp_flatpak_runtime_get_sdk_name (GBP_FLATPAK_RUNTIME (runtime));
   branch = gbp_flatpak_runtime_get_branch (GBP_FLATPAK_RUNTIME (runtime));
 
+  if (GBP_IS_FLATPAK_MANIFEST (config))
+    sdk_extensions = gbp_flatpak_manifest_get_sdk_extensions (GBP_FLATPAK_MANIFEST (config));
+
   /*
    * If we got here by using a non-flatpak configuration, then there is a
    * chance we don't have a valid app-id.
    */
-  if (dzl_str_empty0 (app_id))
+  if (ide_str_empty0 (app_id))
     app_id = "com.example.App";
 
   if (platform == NULL && sdk == NULL)
@@ -351,6 +360,7 @@ register_build_init_stage (GbpFlatpakPipelineAddin  *self,
 
   ide_subprocess_launcher_push_argv (launcher, "flatpak");
   ide_subprocess_launcher_push_argv (launcher, "build-init");
+  ide_subprocess_launcher_push_argv (launcher, "--type=app");
   ide_subprocess_launcher_push_argv (launcher, arch);
   ide_subprocess_launcher_push_argv (launcher, staging_dir);
   ide_subprocess_launcher_push_argv (launcher, app_id);
@@ -358,6 +368,17 @@ register_build_init_stage (GbpFlatpakPipelineAddin  *self,
   ide_subprocess_launcher_push_argv (launcher, platform);
   ide_subprocess_launcher_push_argv (launcher, branch);
 
+  if (sdk_extensions != NULL)
+    {
+      for (guint i = 0; sdk_extensions[i]; i++)
+        {
+          g_auto(GStrv) split = g_strsplit (sdk_extensions[i], "/", 2);
+          g_autofree char *arg = g_strdup_printf ("--sdk-extension=%s", split[0]);
+
+          ide_subprocess_launcher_push_argv (launcher, arg);
+        }
+    }
+
   stage = g_object_new (IDE_TYPE_PIPELINE_STAGE_LAUNCHER,
                         "name", _("Preparing build directory"),
                         "launcher", launcher,
@@ -615,7 +636,7 @@ build_bundle_notify_completed (IdePipelineStage *stage,
   if (ide_pipeline_stage_get_completed (stage))
     {
       g_autoptr(GFile) file = g_file_new_for_path (dest_path);
-      dzl_file_manager_show (file, NULL);
+      ide_file_manager_show (file, NULL);
     }
 }
 
@@ -770,7 +791,7 @@ pipeline_addin_iface_init (IdePipelineAddinInterface *iface)
   iface->unload = gbp_flatpak_pipeline_addin_unload;
 }
 
-G_DEFINE_TYPE_WITH_CODE (GbpFlatpakPipelineAddin, gbp_flatpak_pipeline_addin, IDE_TYPE_OBJECT,
+G_DEFINE_FINAL_TYPE_WITH_CODE (GbpFlatpakPipelineAddin, gbp_flatpak_pipeline_addin, IDE_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (IDE_TYPE_PIPELINE_ADDIN,
                                                 pipeline_addin_iface_init))
 
