@@ -47,7 +47,7 @@ struct _GbpFlatpakRuntimeProvider
 static void runtime_provider_iface_init (IdeRuntimeProviderInterface *iface);
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (GbpFlatpakRuntimeProvider, gbp_flatpak_runtime_provider, IDE_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (IDE_TYPE_RUNTIME_PROVIDER, runtime_provider_iface_init))
+                               G_IMPLEMENT_INTERFACE (IDE_TYPE_RUNTIME_PROVIDER, runtime_provider_iface_init))
 
 static void
 gbp_flatpak_runtime_provider_dispose (GObject *object)
@@ -91,13 +91,15 @@ on_runtime_added_cb (GbpFlatpakRuntimeProvider *self,
   const gchar *metadata;
   gboolean is_extension;
 
+  IDE_ENTRY;
+
   g_assert (GBP_IS_FLATPAK_RUNTIME_PROVIDER (self));
   g_assert (info != NULL);
   g_assert (IPC_IS_FLATPAK_SERVICE (service));
   g_assert (g_variant_is_of_type (info, RUNTIME_VARIANT_TYPE));
 
   if (self->runtimes == NULL)
-    return;
+    IDE_EXIT;
 
   if (!runtime_variant_parse (info,
                               &name, &arch, &branch,
@@ -105,15 +107,16 @@ on_runtime_added_cb (GbpFlatpakRuntimeProvider *self,
                               &deploy_dir,
                               &metadata,
                               &is_extension))
-    return;
+    IDE_EXIT;
 
   /* Ignore extensions for now */
   if (is_extension)
-    return;
+    IDE_EXIT;
 
   context = ide_object_ref_context (IDE_OBJECT (self));
   manager = ide_runtime_manager_from_context (context);
-  runtime = gbp_flatpak_runtime_new (name,
+  runtime = gbp_flatpak_runtime_new (IDE_OBJECT (self),
+                                     name,
                                      arch,
                                      branch,
                                      sdk_name,
@@ -121,9 +124,14 @@ on_runtime_added_cb (GbpFlatpakRuntimeProvider *self,
                                      deploy_dir,
                                      metadata,
                                      is_extension);
+
+  g_debug ("Discovered Flatpak runtime %s/%s/%s using SDK %s//%s from %s",
+           name, arch, branch, sdk_name, sdk_branch, deploy_dir);
+
   g_ptr_array_add (self->runtimes, g_object_ref (runtime));
-  ide_object_append (IDE_OBJECT (self), IDE_OBJECT (runtime));
   ide_runtime_manager_add (manager, IDE_RUNTIME (runtime));
+
+  IDE_EXIT;
 }
 
 static void
@@ -272,8 +280,9 @@ gbp_flatpak_runtime_provider_bootstrap_install_cb (GObject      *object,
   else if (!(runtime = find_runtime (self, state->runtime_id)))
     ide_task_return_new_error (task,
                                G_IO_ERROR,
-                               G_IO_ERROR_CANCELLED,
-                               "Operation was cancelled");
+                               G_IO_ERROR_NOT_FOUND,
+                               "Failed to locate %s after installation",
+                               state->runtime_id);
   else
     ide_task_return_pointer (task, g_steal_pointer (&runtime), g_object_unref);
 
@@ -330,8 +339,9 @@ gbp_flatpak_runtime_provider_bootstrap_complete (gpointer data)
       if (!(runtime = find_runtime (self, state->runtime_id)))
         ide_task_return_new_error (task,
                                    G_IO_ERROR,
-                                   G_IO_ERROR_CANCELLED,
-                                   "Operation was cancelled");
+                                   G_IO_ERROR_NOT_FOUND,
+                                   "Failed to locate runtime %s",
+                                   state->runtime_id);
       else
         ide_task_return_pointer (task, g_steal_pointer (&runtime), g_object_unref);
     }
