@@ -20,10 +20,13 @@
 
 #include "config.h"
 
-#include <libide-tree.h>
 #include <libpeas/peas.h>
 
+#include <libide-tree.h>
+#include <libide-editor.h>
+
 #include "gbp-grep-tree-addin.h"
+#include "gbp-grep-workspace-addin.h"
 
 _IDE_EXTERN void
 _gbp_grep_register_types (PeasObjectModule *module)
@@ -31,4 +34,7 @@ _gbp_grep_register_types (PeasObjectModule *module)
   peas_object_module_register_extension_type (module,
                                               IDE_TYPE_TREE_ADDIN,
                                               GBP_TYPE_GREP_TREE_ADDIN);
+  peas_object_module_register_extension_type (module,
+                                              IDE_TYPE_WORKSPACE_ADDIN,
+                                              GBP_TYPE_GREP_WORKSPACE_ADDIN);
 }
