@@ -20,8 +20,11 @@
 
 #define G_LOG_DOMAIN "gbp-notification-addin"
 
+#include "config.h"
+
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+
 #include <libide-foundry.h>
 
 #include "ide-notification-addin.h"
@@ -47,6 +50,22 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (IdeNotificationAddin,
                                IDE_TYPE_OBJECT,
                                G_IMPLEMENT_INTERFACE (IDE_TYPE_PIPELINE_ADDIN, addin_iface_init))
 
+static gboolean
+title_with_default (GBinding     *binding,
+                    const GValue *from_value,
+                    GValue       *to_value,
+                    gpointer      user_data)
+{
+  g_autofree char *str = g_strstrip (g_value_dup_string (from_value));
+
+  if (ide_str_empty0 (str))
+    g_value_set_static_string (to_value, _("Building…"));
+  else
+    g_value_take_string (to_value, g_steal_pointer (&str));
+
+  return TRUE;
+}
+
 static gboolean
 should_supress_message (IdeNotificationAddin *self,
                         const gchar          *message)
@@ -55,7 +74,7 @@ should_supress_message (IdeNotificationAddin *self,
   g_assert (message != NULL);
 
   if (self->last_msg_body == NULL ||
-      !dzl_str_equal0 (self->last_msg_body, message) ||
+      !ide_str_equal0 (self->last_msg_body, message) ||
       self->last_time + GRACE_PERIOD_USEC < g_get_monotonic_time ())
     {
       g_free (self->last_msg_body);
@@ -110,7 +129,11 @@ ide_notification_addin_notify (IdeNotificationAddin *self,
       msg_body = g_strdup_printf (_("Project “%s” failed to build"), project_name);
     }
 
+#ifdef DEVELOPMENT_BUILD
+  icon = g_themed_icon_new ("org.gnome.Builder.Devel-symbolic");
+#else
   icon = g_themed_icon_new ("org.gnome.Builder-symbolic");
+#endif
 
   notification = g_notification_new (msg_title);
   g_notification_set_body (notification, msg_body);
@@ -155,7 +178,9 @@ ide_notification_addin_pipeline_started (IdeNotificationAddin *self,
     {
       /* Setup new in-app notification */
       self->notif = ide_notification_new ();
-      g_object_bind_property (pipeline, "message", self->notif, "title", G_BINDING_SYNC_CREATE);
+      g_object_bind_property_full (pipeline, "message", self->notif, "title",
+                                   G_BINDING_SYNC_CREATE,
+                                   title_with_default, NULL, NULL, NULL);
       ide_notification_attach (self->notif, IDE_OBJECT (pipeline));
 
       /* Withdraw previous shell notification (it's now invalid) */
