@@ -22,13 +22,12 @@
 
 #include "config.h"
 
-#include <libide-editor.h>
-#include <libide-projects.h>
 #include <vte/vte.h>
 
-#ifdef ENABLE_LIBPORTAL
-# include <libportal/portal-gtk3.h>
-#endif
+#include <libide-editor.h>
+
+#include <libide-gtk.h>
+#include <libide-projects.h>
 
 #include "gbp-project-tree-private.h"
 #include "gbp-rename-file-popover.h"
@@ -42,6 +41,38 @@ typedef struct
   guint        needs_collapse : 1;
 } NewState;
 
+typedef struct
+{
+  IdeWorkspace  *workspace;
+  PanelPosition *position;
+} ClosedPosition;
+
+typedef struct
+{
+  GFile     *src;
+  GFile     *dst;
+  GPtrArray *positions;
+} RenameState;
+
+static void
+closed_position_free (ClosedPosition *state)
+{
+  g_clear_object (&state->workspace);
+  g_clear_object (&state->position);
+  g_slice_free (ClosedPosition, state);
+}
+
+static void
+rename_state_free (RenameState *state)
+{
+  g_clear_object (&state->dst);
+  g_clear_object (&state->src);
+  g_clear_pointer (&state->positions, g_ptr_array_unref);
+  g_slice_free (RenameState, state);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (RenameState, rename_state_free)
+
 static void
 new_state_free (NewState *state)
 {
@@ -50,6 +81,99 @@ new_state_free (NewState *state)
   g_slice_free (NewState, state);
 }
 
+static void
+rename_save_cb (GObject      *object,
+                GAsyncResult *result,
+                gpointer      user_data)
+{
+  IdeBuffer *buffer = (IdeBuffer *)object;
+  g_autoptr(IdePage) page = user_data;
+  g_autoptr(GError) error = NULL;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_BUFFER (buffer));
+  g_assert (G_IS_ASYNC_RESULT (result));
+  g_assert (IDE_IS_PAGE (page));
+
+  if (!ide_buffer_save_file_finish (buffer, result, &error))
+    g_warning ("Failed to save file: %s", error->message);
+
+  panel_widget_close (PANEL_WIDGET (page));
+}
+
+static void
+rename_load_cb (GObject      *object,
+                GAsyncResult *result,
+                gpointer      user_data)
+{
+  IdeBufferManager *buffer_manager = (IdeBufferManager *)object;
+  g_autoptr(RenameState) state = user_data;
+  g_autoptr(IdeBuffer) buffer = NULL;
+  g_autoptr(GError) error = NULL;
+
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_BUFFER_MANAGER (buffer_manager));
+  g_assert (G_IS_ASYNC_RESULT (result));
+  g_assert (state != NULL);
+  g_assert (state->positions != NULL);
+
+  if ((buffer = ide_buffer_manager_load_file_finish (buffer_manager, result, &error)))
+    {
+      for (guint i = 0; i < state->positions->len; i++)
+        {
+          ClosedPosition *cp = g_ptr_array_index (state->positions, i);
+          GtkWidget *page = ide_editor_page_new (buffer);
+
+          g_assert (cp != NULL);
+          g_assert (IDE_IS_WORKSPACE (cp->workspace));
+          g_assert (PANEL_IS_POSITION (cp->position));
+
+          ide_workspace_add_page (cp->workspace, IDE_PAGE (page), cp->position);
+        }
+    }
+
+  IDE_EXIT;
+}
+
+static void
+rename_state_close_matching_and_save_position (IdePage *page,
+                                               gpointer user_data)
+{
+  RenameState *state = user_data;
+  GFile *this_file;
+
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_PAGE (page));
+
+  if (!IDE_IS_EDITOR_PAGE (page))
+    return;
+
+  if (!(this_file = ide_editor_page_get_file (IDE_EDITOR_PAGE (page))))
+    return;
+
+  g_assert (state != NULL);
+  g_assert (G_IS_FILE (state->src));
+  g_assert (G_IS_FILE (state->dst));
+  g_assert (G_IS_FILE (this_file));
+
+  if (g_file_equal (this_file, state->src))
+    {
+      IdeBuffer *buffer = ide_editor_page_get_buffer (IDE_EDITOR_PAGE (page));
+      ClosedPosition *cp = g_slice_new0 (ClosedPosition);
+
+      cp->workspace = g_object_ref (ide_widget_get_workspace (GTK_WIDGET (page)));
+      cp->position = panel_widget_get_position (PANEL_WIDGET (page));
+      g_ptr_array_add (state->positions, cp);
+
+      ide_buffer_save_file_async (buffer, NULL, NULL,
+                                  NULL,
+                                  rename_save_cb,
+                                  g_object_ref (page));
+    }
+}
+
 static void
 new_action_completed_cb (GObject      *object,
                          GAsyncResult *result,
@@ -77,7 +201,14 @@ new_action_completed_cb (GObject      *object,
         return;
 
       if (state->file != NULL)
-        ide_workbench_open_async (workbench, state->file, "editor", 0, NULL, NULL, NULL);
+        ide_workbench_open_async (workbench,
+                                  state->file,
+                                  "editorui",
+                                  0,
+                                  NULL,
+                                  NULL,
+                                  NULL,
+                                  NULL);
     }
 }
 
@@ -168,7 +299,7 @@ gbp_project_tree_pane_actions_new_cb (GObject      *object,
   else
     g_assert_not_reached ();
 
-  gtk_widget_destroy (GTK_WIDGET (popover));
+  gtk_popover_popdown (GTK_POPOVER (popover));
 }
 
 static void
@@ -218,7 +349,6 @@ gbp_project_tree_pane_actions_new (GbpProjectTreePane *self,
                           "position", GTK_POS_RIGHT,
                           NULL);
 
-
   state = g_slice_new0 (NewState);
   state->needs_collapse = !ide_tree_node_expanded (self->tree, selected);
   state->file_type = file_type;
@@ -237,10 +367,9 @@ gbp_project_tree_pane_actions_new (GbpProjectTreePane *self,
 }
 
 static void
-close_matching_pages (GtkWidget *widget,
-                      gpointer   user_data)
+close_matching_pages (IdePage  *page,
+                      gpointer  user_data)
 {
-  IdePage *page = (IdePage *)widget;
   GFile *file = user_data;
   GFile *this_file;
 
@@ -250,46 +379,59 @@ close_matching_pages (GtkWidget *widget,
   if (!IDE_IS_EDITOR_PAGE (page))
     return;
 
-  this_file = ide_editor_page_get_file (IDE_EDITOR_PAGE (page));
-  if (this_file == NULL)
+  if (!(this_file = ide_editor_page_get_file (IDE_EDITOR_PAGE (page))))
     return;
 
   if (g_file_equal (this_file, file))
     {
       IdeBuffer *buffer = ide_editor_page_get_buffer (IDE_EDITOR_PAGE (page));
-      ide_buffer_save_file_async (buffer, NULL, NULL, NULL, NULL, NULL);
-      gtk_widget_destroy (widget);
+      ide_buffer_save_file_async (buffer, NULL, NULL,
+                                  NULL,
+                                  rename_save_cb,
+                                  g_object_ref (page));
     }
 }
 
-#define DEFINE_ACTION_HANDLER(short_name, BODY)                       \
-static void                                                           \
-gbp_project_tree_pane_actions_##short_name (GSimpleAction *action,    \
-                                            GVariant      *param,     \
-                                            gpointer       user_data) \
-{                                                                     \
-  GbpProjectTreePane *self = user_data;                               \
-                                                                      \
-  g_assert (G_IS_SIMPLE_ACTION (action));                             \
-  g_assert (GBP_IS_PROJECT_TREE_PANE (self));                         \
-                                                                      \
-  BODY                                                                \
+static void
+gbp_project_tree_pane_actions_new_file (GSimpleAction *action,
+                                        GVariant      *param,
+                                        gpointer       user_data)
+{
+  GbpProjectTreePane *self = user_data;
+
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
+
+  gbp_project_tree_pane_actions_new (self, G_FILE_TYPE_REGULAR);
 }
 
-DEFINE_ACTION_HANDLER (new_file, {
-  gbp_project_tree_pane_actions_new (self, G_FILE_TYPE_REGULAR);
-});
+static void
+gbp_project_tree_pane_actions_new_folder (GSimpleAction *action,
+                                          GVariant      *param,
+                                          gpointer       user_data)
+{
+  GbpProjectTreePane *self = user_data;
+
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
 
-DEFINE_ACTION_HANDLER (new_folder, {
   gbp_project_tree_pane_actions_new (self, G_FILE_TYPE_DIRECTORY);
-});
+}
 
-DEFINE_ACTION_HANDLER (open, {
+static void
+gbp_project_tree_pane_actions_open (GSimpleAction *action,
+                                    GVariant      *param,
+                                    gpointer       user_data)
+{
+  GbpProjectTreePane *self = user_data;
   IdeProjectFile *project_file;
   g_autoptr(GFile) file = NULL;
   IdeWorkbench *workbench;
   IdeTreeNode *selected;
 
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
+
   if (!(selected = ide_tree_get_selected_node (self->tree)) ||
       !ide_tree_node_holds (selected, IDE_TYPE_PROJECT_FILE) ||
       !(project_file = ide_tree_node_get_item (selected)))
@@ -302,8 +444,11 @@ DEFINE_ACTION_HANDLER (open, {
                             file,
                             NULL,
                             IDE_BUFFER_OPEN_FLAGS_NONE,
-                            NULL, NULL, NULL);
-});
+                            NULL,
+                            NULL,
+                            NULL,
+                            NULL);
+}
 
 static void
 gbp_project_tree_pane_actions_rename_cb (GObject      *object,
@@ -311,14 +456,40 @@ gbp_project_tree_pane_actions_rename_cb (GObject      *object,
                                          gpointer      user_data)
 {
   IdeProject *project = (IdeProject *)object;
-  g_autoptr(GbpProjectTreePane) self = user_data;
+  g_autoptr(RenameState) state = user_data;
   g_autoptr(GError) error = NULL;
+  IdeBufferManager *buffer_manager;
+  IdeContext *context;
+  GFile *file;
+
+  IDE_ENTRY;
 
   g_assert (IDE_IS_MAIN_THREAD ());
-  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
+  g_assert (state != NULL);
+  g_assert (G_IS_FILE (state->src));
+  g_assert (G_IS_FILE (state->dst));
+  g_assert (state->positions != NULL);
 
   if (!ide_project_rename_file_finish (project, result, &error))
     g_warning ("Failed to rename file: %s", error->message);
+
+  context = ide_object_get_context (IDE_OBJECT (project));
+  buffer_manager = ide_buffer_manager_from_context (context);
+
+  if (state->positions->len == 0)
+    IDE_EXIT;
+
+  file = state->dst;
+
+  ide_buffer_manager_load_file_async (buffer_manager,
+                                      file,
+                                      IDE_BUFFER_OPEN_FLAGS_NONE,
+                                      NULL,
+                                      NULL,
+                                      rename_load_cb,
+                                      g_steal_pointer (&state));
+
+  IDE_EXIT;
 }
 
 static void
@@ -330,6 +501,8 @@ gbp_project_tree_pane_actions_rename_display_cb (GObject      *object,
   g_autoptr(GbpProjectTreePane) self = user_data;
   g_autoptr(GError) error = NULL;
   g_autoptr(GFile) dst = NULL;
+  IdeWorkbench *workbench;
+  RenameState *state;
   IdeProject *project;
   IdeContext *context;
   GFile *src;
@@ -338,31 +511,48 @@ gbp_project_tree_pane_actions_rename_display_cb (GObject      *object,
   g_assert (GBP_IS_PROJECT_TREE_PANE (self));
 
   if (!(dst = gbp_rename_file_popover_display_finish (popover, result, &error)))
-    goto destroy;
+    goto done;
 
   src = gbp_rename_file_popover_get_file (popover);
   context = ide_widget_get_context (GTK_WIDGET (self));
   project = ide_project_from_context (context);
+  workbench = ide_workbench_from_context (context);
+
+  state = g_slice_new0 (RenameState);
+  state->src = g_object_ref (src);
+  state->dst = g_object_ref (dst);
+  state->positions = g_ptr_array_new_with_free_func ((GDestroyNotify)closed_position_free);
+
+  ide_workbench_foreach_page (workbench,
+                              rename_state_close_matching_and_save_position,
+                              state);
 
   ide_project_rename_file_async (project,
                                  src,
                                  dst,
                                  NULL,
                                  gbp_project_tree_pane_actions_rename_cb,
-                                 g_object_ref (self));
+                                 g_steal_pointer (&state));
 
-destroy:
-  gtk_widget_destroy (GTK_WIDGET (popover));
+done:
+  gtk_popover_popdown (GTK_POPOVER (popover));
 }
 
-DEFINE_ACTION_HANDLER (rename, {
+static void
+gbp_project_tree_pane_actions_rename (GSimpleAction *action,
+                                      GVariant      *param,
+                                      gpointer       user_data)
+{
+  GbpProjectTreePane *self = user_data;
   IdeProjectFile *project_file;
   g_autoptr(GFile) file = NULL;
   GbpRenameFilePopover *popover;
-  IdeWorkbench *workbench;
   IdeTreeNode *selected;
   gboolean is_dir;
 
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
+
   if (!(selected = ide_tree_get_selected_node (self->tree)) ||
       !ide_tree_node_holds (selected, IDE_TYPE_PROJECT_FILE) ||
       !(project_file = ide_tree_node_get_item (selected)))
@@ -370,8 +560,6 @@ DEFINE_ACTION_HANDLER (rename, {
 
   is_dir = ide_project_file_is_directory (project_file);
   file = ide_project_file_ref_file (project_file);
-  workbench = ide_widget_get_workbench (GTK_WIDGET (self->tree));
-  ide_workbench_foreach_page (workbench, close_matching_pages, file);
 
   popover = g_object_new (GBP_TYPE_RENAME_FILE_POPOVER,
                           "position", GTK_POS_LEFT,
@@ -385,7 +573,7 @@ DEFINE_ACTION_HANDLER (rename, {
                                          NULL,
                                          gbp_project_tree_pane_actions_rename_display_cb,
                                          g_object_ref (self));
-});
+}
 
 static void
 gbp_project_tree_pane_actions_trash_cb (GObject      *object,
@@ -409,12 +597,20 @@ gbp_project_tree_pane_actions_trash_cb (GObject      *object,
     ide_tree_node_remove (parent, node);
 }
 
-DEFINE_ACTION_HANDLER (trash, {
+static void
+gbp_project_tree_pane_actions_trash (GSimpleAction *action,
+                                     GVariant      *param,
+                                     gpointer       user_data)
+{
+  GbpProjectTreePane *self = user_data;
   IdeProjectFile *project_file;
   g_autoptr(GFile) file = NULL;
   IdeWorkbench *workbench;
   IdeTreeNode *selected;
 
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
+
   if (!(selected = ide_tree_get_selected_node (self->tree)) ||
       !ide_tree_node_holds (selected, IDE_TYPE_PROJECT_FILE) ||
       !(project_file = ide_tree_node_get_item (selected)))
@@ -428,29 +624,45 @@ DEFINE_ACTION_HANDLER (trash, {
                                 NULL,
                                 gbp_project_tree_pane_actions_trash_cb,
                                 g_object_ref (selected));
-});
+}
 
-DEFINE_ACTION_HANDLER (open_containing_folder, {
+static void
+gbp_project_tree_pane_actions_open_containing_folder (GSimpleAction *action,
+                                                      GVariant      *param,
+                                                      gpointer       user_data)
+{
+  GbpProjectTreePane *self = user_data;
   IdeProjectFile *project_file;
   g_autoptr(GFile) file = NULL;
   IdeTreeNode *selected;
 
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
+
   if (!(selected = ide_tree_get_selected_node (self->tree)) ||
       !ide_tree_node_holds (selected, IDE_TYPE_PROJECT_FILE) ||
       !(project_file = ide_tree_node_get_item (selected)))
     return;
 
   file = ide_project_file_ref_file (project_file);
-  dzl_file_manager_show (file, NULL);
-});
+  ide_file_manager_show (file, NULL);
+}
 
-DEFINE_ACTION_HANDLER (open_with_hint, {
+static void
+gbp_project_tree_pane_actions_open_with_hint (GSimpleAction *action,
+                                              GVariant      *param,
+                                              gpointer       user_data)
+{
+  GbpProjectTreePane *self = user_data;
   IdeProjectFile *project_file;
   g_autoptr(GFile) file = NULL;
   IdeWorkbench *workbench;
   IdeTreeNode *selected;
   const gchar *hint;
 
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (GBP_IS_PROJECT_TREE_PANE (self));
+
   if (!(selected = ide_tree_get_selected_node (self->tree)) ||
       !ide_tree_node_holds (selected, IDE_TYPE_PROJECT_FILE) ||
       !(project_file = ide_tree_node_get_item (selected)) ||
@@ -464,38 +676,11 @@ DEFINE_ACTION_HANDLER (open_with_hint, {
                             file,
                             hint,
                             IDE_BUFFER_OPEN_FLAGS_NONE,
-                            NULL, NULL, NULL);
-});
-
-#ifdef ENABLE_LIBPORTAL
-DEFINE_ACTION_HANDLER (open_with_external, {
-  IdeProjectFile *project_file;
-  g_autoptr(XdpPortal) portal = NULL;
-  g_autoptr(GFile) file = NULL;
-  g_autofree gchar *uri = NULL;
-  IdeTreeNode *selected;
-  XdpParent *parent;
-  GtkWidget *toplevel;
-
-  if (!(selected = ide_tree_get_selected_node (self->tree)) ||
-      !ide_tree_node_holds (selected, IDE_TYPE_PROJECT_FILE) ||
-      !(project_file = ide_tree_node_get_item (selected)))
-    return;
-
-  toplevel = gtk_widget_get_ancestor (GTK_WIDGET (self->tree), GTK_TYPE_WINDOW);
-  file = ide_project_file_ref_file (project_file);
-  uri = g_file_get_uri (file);
-
-  portal = xdp_portal_new ();
-  parent = xdp_parent_new_gtk (GTK_WINDOW (toplevel));
-  xdp_portal_open_uri (portal,
-                       parent,
-                       uri,
-                       XDP_OPEN_URI_FLAG_ASK | XDP_OPEN_URI_FLAG_WRITABLE,
-                       NULL, NULL, NULL);
-  xdp_parent_free (parent);
-});
-#endif
+                            NULL,
+                            NULL,
+                            NULL,
+                            NULL);
+}
 
 /* Based on gdesktopappinfo.c in GIO */
 static gchar *
@@ -582,9 +767,6 @@ static const GActionEntry entries[] = {
   { "new-folder", gbp_project_tree_pane_actions_new_folder },
   { "open", gbp_project_tree_pane_actions_open },
   { "open-with-hint", gbp_project_tree_pane_actions_open_with_hint, "s" },
-#ifdef ENABLE_LIBPORTAL
-  { "open-with-external", gbp_project_tree_pane_actions_open_with_external },
-#endif
   { "open-containing-folder", gbp_project_tree_pane_actions_open_containing_folder },
   { "open-in-terminal", gbp_project_tree_pane_actions_open_in_terminal },
   { "rename", gbp_project_tree_pane_actions_rename },
@@ -612,13 +794,31 @@ _gbp_project_tree_pane_init_actions (GbpProjectTreePane *self)
                                    self);
   g_action_map_add_action (G_ACTION_MAP (actions), ignored_action);
   g_action_map_add_action (G_ACTION_MAP (actions), sort_action);
-  gtk_widget_insert_action_group (GTK_WIDGET (self->tree),
+  gtk_widget_insert_action_group (GTK_WIDGET (self),
                                   "project-tree",
                                   G_ACTION_GROUP (actions));
 
+  self->actions = g_object_ref (G_ACTION_GROUP (actions));
+
   _gbp_project_tree_pane_update_actions (self);
 }
 
+G_GNUC_NULL_TERMINATED static void
+action_map_set (GActionMap *map,
+                const char *name,
+                const char *first_property,
+                ...)
+{
+  GAction *action;
+  va_list args;
+
+  action = g_action_map_lookup_action (map, name);
+
+  va_start (args, first_property);
+  g_object_set_valist (G_OBJECT (action), first_property, args);
+  va_end (args);
+}
+
 void
 _gbp_project_tree_pane_update_actions (GbpProjectTreePane *self)
 {
@@ -643,33 +843,28 @@ _gbp_project_tree_pane_update_actions (GbpProjectTreePane *self)
         }
     }
 
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "new-file",
-                             "enabled", is_file,
-                             NULL);
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "new-folder",
-                             "enabled", is_file,
-                             NULL);
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "trash",
-                             "enabled", is_file,
-                             NULL);
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "rename",
-                             "enabled", is_file,
-                             NULL);
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "open",
-                             "enabled", is_file && !is_dir,
-                             NULL);
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "open-with-hint",
-                             "enabled", is_file,
-                             NULL);
-#ifdef ENABLE_LIBPORTAL
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "open-with-external",
-                             "enabled", is_file,
-                             NULL);
-#endif
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "open-containing-folder",
-                             "enabled", is_file,
-                             NULL);
-  dzl_gtk_widget_action_set (GTK_WIDGET (self->tree), "project-tree", "open-in-terminal",
-                             "enabled", is_file,
-                             NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "new-file",
+                  "enabled", is_file,
+                  NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "new-folder",
+                  "enabled", is_file,
+                  NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "trash",
+                  "enabled", is_file,
+                  NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "rename",
+                  "enabled", is_file,
+                  NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "open",
+                  "enabled", is_file && !is_dir,
+                  NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "open-with-hint",
+                  "enabled", is_file,
+                  NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "open-containing-folder",
+                  "enabled", is_file,
+                  NULL);
+  action_map_set (G_ACTION_MAP (self->actions), "open-in-terminal",
+                  "enabled", is_file,
+                  NULL);
 }
