@@ -31,10 +31,12 @@ struct _GbpQuickHighlightEditorPageAddin
 {
   GObject                 parent_instance;
 
+  GSettings              *settings;
+
   IdeEditorPage          *view;
 
-  DzlSignalGroup         *buffer_signals;
-  DzlSignalGroup         *search_signals;
+  IdeSignalGroup         *buffer_signals;
+  IdeSignalGroup         *search_signals;
   GtkSourceSearchContext *search_context;
 
   guint                   queued_match_source;
@@ -51,6 +53,8 @@ do_delayed_quick_highlight (GbpQuickHighlightEditorPageAddin *self)
   IdeBuffer *buffer;
   GtkTextIter begin;
   GtkTextIter end;
+  int selection_length;
+  int min_length;
 
   g_assert (GBP_IS_QUICK_HIGHLIGHT_EDITOR_PAGE_ADDIN (self));
   g_assert (self->view != NULL);
@@ -58,7 +62,7 @@ do_delayed_quick_highlight (GbpQuickHighlightEditorPageAddin *self)
   self->queued_match_source = 0;
 
   /*
-   * Get the curretn selection, if any. Short circuit if we find a situation
+   * Get the current selection, if any. Short circuit if we find a situation
    * that should have caused us to cancel the current quick-highlight.
    */
   buffer = ide_editor_page_get_buffer (self->view);
@@ -70,6 +74,14 @@ do_delayed_quick_highlight (GbpQuickHighlightEditorPageAddin *self)
       return G_SOURCE_REMOVE;
     }
 
+  min_length = g_settings_get_int (self->settings, "min-char-selected");
+  selection_length = gtk_text_iter_get_offset (&end) - gtk_text_iter_get_offset (&begin);
+  if (selection_length < min_length)
+    {
+      g_clear_object (&self->search_context);
+      return G_SOURCE_REMOVE;
+    }
+
   /*
    * If the current selection goes across a line, then ignore trying to match
    * anything similar as it's unlikely to be what the user wants.
@@ -125,25 +137,23 @@ do_delayed_quick_highlight (GbpQuickHighlightEditorPageAddin *self)
 
 static void
 buffer_cursor_moved (GbpQuickHighlightEditorPageAddin *self,
-                     const GtkTextIter                *location,
                      IdeBuffer                        *buffer)
 {
   g_assert (GBP_IS_QUICK_HIGHLIGHT_EDITOR_PAGE_ADDIN (self));
-  g_assert (location != NULL);
   g_assert (IDE_IS_BUFFER (buffer));
 
   if (self->has_selection && !self->search_active)
     {
       if (self->queued_match_source == 0)
         self->queued_match_source =
-          gdk_threads_add_idle_full (G_PRIORITY_LOW + 100,
-                                     (GSourceFunc) do_delayed_quick_highlight,
-                                     g_object_ref (self),
-                                     g_object_unref);
+          g_idle_add_full (G_PRIORITY_LOW + 100,
+                           (GSourceFunc) do_delayed_quick_highlight,
+                           g_object_ref (self),
+                           g_object_unref);
     }
   else
     {
-      dzl_clear_source (&self->queued_match_source);
+      g_clear_handle_id (&self->queued_match_source, g_source_remove);
       g_clear_object (&self->search_context);
     }
 }
@@ -180,6 +190,7 @@ buffer_notify_has_selection (GbpQuickHighlightEditorPageAddin *self,
   self->has_selection = gtk_text_buffer_get_has_selection (GTK_TEXT_BUFFER (buffer));
 }
 
+#if 0
 static void
 search_notify_active (GbpQuickHighlightEditorPageAddin *self,
                       GParamSpec                       *pspec,
@@ -191,6 +202,7 @@ search_notify_active (GbpQuickHighlightEditorPageAddin *self,
   self->search_active = ide_editor_search_get_active (search);
   do_delayed_quick_highlight (self);
 }
+#endif
 
 static void
 gbp_quick_highlight_editor_page_addin_load (IdeEditorPageAddin *addin,
@@ -203,32 +215,31 @@ gbp_quick_highlight_editor_page_addin_load (IdeEditorPageAddin *addin,
 
   self->view = view;
 
-  self->buffer_signals = dzl_signal_group_new (IDE_TYPE_BUFFER);
+  self->settings = g_settings_new ("org.gnome.builder.editor");
 
-  dzl_signal_group_connect_swapped (self->buffer_signals,
+  self->buffer_signals = ide_signal_group_new (IDE_TYPE_BUFFER);
+  ide_signal_group_connect_swapped (self->buffer_signals,
                                     "notify::has-selection",
                                     G_CALLBACK (buffer_notify_has_selection),
                                     self);
-
-  dzl_signal_group_connect_swapped (self->buffer_signals,
+  ide_signal_group_connect_swapped (self->buffer_signals,
                                     "notify::style-scheme",
                                     G_CALLBACK (buffer_notify_style_scheme),
                                     self);
-
-  dzl_signal_group_connect_swapped (self->buffer_signals,
+  ide_signal_group_connect_swapped (self->buffer_signals,
                                     "cursor-moved",
                                     G_CALLBACK (buffer_cursor_moved),
                                     self);
+  ide_signal_group_set_target (self->buffer_signals, ide_editor_page_get_buffer (view));
 
-  self->search_signals = dzl_signal_group_new (IDE_TYPE_EDITOR_SEARCH);
-
-  dzl_signal_group_connect_swapped (self->search_signals,
+#if 0
+  self->search_signals = ide_signal_group_new (IDE_TYPE_EDITOR_SEARCH);
+  ide_signal_group_connect_swapped (self->search_signals,
                                     "notify::active",
                                     G_CALLBACK (search_notify_active),
                                     self);
-
-  dzl_signal_group_set_target (self->buffer_signals, ide_editor_page_get_buffer (view));
-  dzl_signal_group_set_target (self->search_signals, ide_editor_page_get_search (view));
+  ide_signal_group_set_target (self->search_signals, ide_editor_page_get_search (view));
+#endif
 }
 
 static void
@@ -241,13 +252,17 @@ gbp_quick_highlight_editor_page_addin_unload (IdeEditorPageAddin *addin,
   g_assert (IDE_IS_EDITOR_PAGE (view));
 
   g_clear_object (&self->search_context);
-  dzl_clear_source (&self->queued_match_source);
+  g_clear_handle_id (&self->queued_match_source, g_source_remove);
 
-  dzl_signal_group_set_target (self->buffer_signals, NULL);
+  ide_signal_group_set_target (self->buffer_signals, NULL);
   g_clear_object (&self->buffer_signals);
 
-  dzl_signal_group_set_target (self->search_signals, NULL);
+#if 0
+  ide_signal_group_set_target (self->search_signals, NULL);
   g_clear_object (&self->search_signals);
+#endif
+
+  g_clear_object (&self->settings);
 
   self->view = NULL;
 }
@@ -260,10 +275,9 @@ editor_view_addin_iface_init (IdeEditorPageAddinInterface *iface)
 }
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (GbpQuickHighlightEditorPageAddin,
-                         gbp_quick_highlight_editor_page_addin,
-                         G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (IDE_TYPE_EDITOR_PAGE_ADDIN,
-                                                editor_view_addin_iface_init))
+                               gbp_quick_highlight_editor_page_addin,
+                               G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (IDE_TYPE_EDITOR_PAGE_ADDIN, editor_view_addin_iface_init))
 
 static void
 gbp_quick_highlight_editor_page_addin_class_init (GbpQuickHighlightEditorPageAddinClass *klass)
