@@ -26,19 +26,15 @@
 #include <libide-gui.h>
 #include <libide-terminal.h>
 
-#include "gbp-terminal-application-addin.h"
-#include "gbp-terminal-preferences-addin.h"
+#include "gbp-terminal-tweaks-addin.h"
 #include "gbp-terminal-workspace-addin.h"
 
 _IDE_EXTERN void
 _gbp_terminal_register_types (PeasObjectModule *module)
 {
   peas_object_module_register_extension_type (module,
-                                              IDE_TYPE_APPLICATION_ADDIN,
-                                              GBP_TYPE_TERMINAL_APPLICATION_ADDIN);
-  peas_object_module_register_extension_type (module,
-                                              IDE_TYPE_PREFERENCES_ADDIN,
-                                              GBP_TYPE_TERMINAL_PREFERENCES_ADDIN);
+                                              IDE_TYPE_TWEAKS_ADDIN,
+                                              GBP_TYPE_TERMINAL_TWEAKS_ADDIN);
   peas_object_module_register_extension_type (module,
                                               IDE_TYPE_WORKSPACE_ADDIN,
                                               GBP_TYPE_TERMINAL_WORKSPACE_ADDIN);
