@@ -22,15 +22,16 @@
 
 #include "config.h"
 
-#include <libide-tree.h>
 #include <libpeas/peas.h>
 
-#include "gbp-test-tree-addin.h"
+#include <libide-gui.h>
+
+#include "gbp-testui-workspace-addin.h"
 
 _IDE_EXTERN void
 _gbp_testui_register_types (PeasObjectModule *module)
 {
   peas_object_module_register_extension_type (module,
-                                              IDE_TYPE_TREE_ADDIN,
-                                              GBP_TYPE_TEST_TREE_ADDIN);
+                                              IDE_TYPE_WORKSPACE_ADDIN,
+                                              GBP_TYPE_TESTUI_WORKSPACE_ADDIN);
 }
