@@ -20,7 +20,7 @@
 
 #pragma once
 
-#include <dazzle.h>
+#include <libide-gui.h>
 
 #include "gbp-todo-model.h"
 
@@ -28,7 +28,7 @@ G_BEGIN_DECLS
 
 #define GBP_TYPE_TODO_PANEL (gbp_todo_panel_get_type())
 
-G_DECLARE_FINAL_TYPE (GbpTodoPanel, gbp_todo_panel, GBP, TODO_PANEL, DzlDockWidget)
+G_DECLARE_FINAL_TYPE (GbpTodoPanel, gbp_todo_panel, GBP, TODO_PANEL, IdePane)
 
 GbpTodoModel *gbp_todo_panel_get_model  (GbpTodoPanel *self);
 void          gbp_todo_panel_set_model  (GbpTodoPanel *self,
